# sage_setup: distribution = sagemath-repl
from typing import Callable, Any
import valgrind


def verify_no_leak(callback: Callable[[], Any],
                   repeat: int = 10000,
                   fuzzy: int = 10,
                   ) -> None:
    """
    Verify that the callback does not generate new definitely lost blocks

    Raises an assertion if the callback leaks memory
    """
    callback()   # warm_up
    initial_blocks = (0, 0, 0, 0)
    valgrind.memcheck_do_leak_check()
    initial_blocks = valgrind.memcheck_count_leak_blocks()
    for _ in range(repeat):
        callback()
    valgrind.memcheck_do_leak_check()
    leak_blocks = valgrind.memcheck_count_leak_blocks()
    leak = leak_blocks[0] - initial_blocks[0]
    if leak < repeat - fuzzy:
        return  # callback did not leak at least once per call
    blocks = round(leak / repeat, 2)
    message = f'{callback} leaked {blocks} block on average ({repeat} iterations)'
    raise AssertionError(message)



# sage_setup: distribution = sagemath-repl
r"""
This file contains some tests that Sage command line options actually
do something.

We test the following command line options:

test.py
/path/to/test.py
test.sage
/path/to/test.sage
test.spyx
/path/to/test.spyx
--advanced
-c
--cython
--dev
--ecl
--experimental
--fixdoctests
--gap
--gdb
--gp
-h
--help
--info
--ipython
--lisp
--maxima
--min
--mwrank
--optional
--preparse
--python
--python3
-q
--R
--root
--rst2ipynb
--ipynb2rst
--sh
--singular
--sqlite3
--standard
--startuptime
-t
-v
--zzfoobar (illegal option)


AUTHORS:

- Jeroen Demeyer (2010-11-20): initial version (:issue:`10300`)
"""
from subprocess import Popen, PIPE
import os
import sys
import select


def test_executable(args, input='', timeout=100.0, pydebug_ignore_warnings=False, **kwds):
    r"""
    Run the program defined by ``args`` using the string ``input`` on
    the standard input.

    INPUT:

    - ``args`` -- list of program arguments, the first being the
      executable

    - ``input`` -- string serving as standard input; usually, this
      should end with a newline

    - ``timeout`` -- if the program produces no output for ``timeout``
      seconds, a :exc:`RuntimeError` is raised

    - ``pydebug_ignore_warnings`` -- boolean. Set the PYTHONWARNINGS environment variable to ignore
      Python warnings when on a Python debug build (`--with-pydebug`, e.g. from building with
      `SAGE_DEBUG=yes`). Debug builds do not install the default warning filters, which can break
      some doctests. Unfortunately the environment variable does not support regex message filters,
      so the filter will catch a bit more than the default filters. Hence we only enable it on debug
      builds.

    - ``**kwds`` -- additional keyword arguments passed to the
      :class:`Popen` constructor

    OUTPUT: a tuple ``(out, err, ret)`` with the standard output,
    standard error and exitcode of the program run.

    EXAMPLES::

        sage: from sage.tests.cmdline import test_executable
        sage: (out, err, ret) = test_executable(["cat"], "Hello World!")
        sage: out
        'Hello World!'
        sage: err
        ''
        sage: ret
        0

    We test the timeout option::

        sage: (out, err, ret) = test_executable(["sleep", "1"], timeout=0.1)
        Traceback (most recent call last):
        ...
        RuntimeError: timeout in test_executable()

    TESTS:

    Run Sage itself with various options::

        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage"], pydebug_ignore_warnings=True)
        sage: out.find(version()) >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage"], "3^33\n", pydebug_ignore_warnings=True)
        sage: out.find(version()) >= 0
        True
        sage: out.find("5559060566555523") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage", "-q"], "3^33\n", pydebug_ignore_warnings=True)
        sage: out.find(version()) >= 0
        False
        sage: out.find("5559060566555523") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage", "-c", "print(3^33)"])
        sage: print(out)
        5559060566555523
        sage: err
        ''
        sage: ret
        0

        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage", "--min", "-c", "print(3^33)"])
        sage: print(out)
        5559060566555523
        sage: err
        ''
        sage: ret
        0

        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage", "--startuptime"])
        sage: out.find("Slowest module import") >= 0
        True
        sage: err
        ''
        sage: ret
        0

    Test help::

        sage: (out, err, ret) = test_executable(["sage", "-h"])
        sage: out.find("evaluate cmd as sage") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: (out, err, ret) = test_executable(["sage", "--help"])
        sage: out.find("evaluate cmd as sage") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: (out, err, ret) = test_executable(["sage", "--advanced"])
        sage: out.find("run the Sage cleaner.") >= 0
        True
        sage: err
        ''
        sage: ret
        0
        sage: out.find("print the Sage root directory") >= 0 # optional - sage_spkg
        True
        sage: out.find("regular expression search through the Sage") >= 0 # optional - sage_spkg
        True

    Basic information about the Sage installation::

        sage: (out, err, ret) = test_executable(["sage", "-v"])
        sage: out.find(version()) >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # optional - sage_spkg
        sage: (out, err, ret) = test_executable(["sage", "--root"])
        sage: len(out) >= 2
        True
        sage: err
        ''
        sage: ret
        0

    Test ``sage --info [packages]``::

        sage: # optional - sage_spkg
        sage: out, err, ret = test_executable(["sage", "--info", "sqlite"])
        sage: print(out)
        sqlite...
        SQLite is a software library that implements a self-contained,
        serverless, zero-configuration, transactional SQL database engine.
        ...
        sage: err
        ''
        sage: ret
        0

    Test ``sage-run`` on a Python file, both with an absolute and with a relative path::

        sage: import tempfile
        sage: with tempfile.TemporaryDirectory() as dir:
        ....:     name = 'python_test_file.py'
        ....:     fullname = os.path.join(dir, name)
        ....:     with open(fullname, 'w') as F:
        ....:         _ = F.write("print(3^33)\n")
        ....:     test_executable(["sage", fullname])
        ....:     test_executable(["sage", name], cwd=dir)
        ('34\n', '', 0)
        ('34\n', '', 0)

    The same as above, but now with a ``.sage`` file.  This indirectly
    also tests the preparser::

        sage: import tempfile
        sage: with tempfile.TemporaryDirectory() as dir:  # long time
        ....:     name = 'sage_test_file.sage'
        ....:     fullname = os.path.join(dir, name)
        ....:     with open(fullname, 'w') as F:
        ....:         _ = F.write("k.<a> = GF(5^3); print(a^124)\n")
        ....:     test_executable(["sage", fullname])
        ....:     test_executable(["sage", name], cwd=dir)
        ('1\n', '', 0)
        ('1\n', '', 0)

    Test running a ``.spyx`` file::

        sage: import tempfile
        sage: with tempfile.TemporaryDirectory() as dir:  # long time
        ....:     name = 'sage_test_file.spyx'
        ....:     fullname = os.path.join(dir, name)
        ....:     with open(fullname, 'w') as F:
        ....:         _ = F.write("from cysignals.signals cimport *\n")
        ....:         _ = F.write("from sage.rings.integer cimport Integer\n")
        ....:         _ = F.write("cdef long i, s = 0\n")
        ....:         _ = F.write("sig_on()\n")
        ....:         _ = F.write("for i in range(5): s += i\n")
        ....:         _ = F.write("sig_off()\n")
        ....:         _ = F.write("print(Integer(s))")
        ....:     test_executable(["sage", fullname], pydebug_ignore_warnings=True)
        ....:     test_executable(["sage", name], cwd=dir, pydebug_ignore_warnings=True)
        ('10\n', 'Compiling .../sage_test_file.spyx...\n', 0)
        ('10\n', 'Compiling sage_test_file.spyx...\n', 0)

    Testing ``sage --preparse FILE`` and ``sage -t FILE``.  First create
    a file and preparse it::

        sage: s = "# -*- coding: utf-8 -*-\n'''This is a test file.\nAnd I am its doctest'''\ndef my_add(a):\n    '''\n    Add 2 to a.\n\n        EXAMPLES::\n\n            sage: my_add(2)\n            4\n        '''\n    return a + 2\n"
        sage: script = os.path.join(tmp_dir(), 'my_script.sage')
        sage: script_py = script + '.py'
        sage: F = open(script, 'w')
        sage: _ = F.write(s)
        sage: F.close()
        sage: (out, err, ret) = test_executable(["sage", "--preparse", script])
        sage: ret
        0
        sage: os.path.isfile(script_py)
        True

    Now test my_script.sage and the preparsed version my_script.sage.py::

        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage", "-t", "--optional=sage", script])
        sage: ret
        0
        sage: out.find("All tests passed!") >= 0
        True
        sage: (out, err, ret) = test_executable([
        ....:     "sage", "-t", "--optional=sage", script_py])
        sage: ret
        0
        sage: out.find("All tests passed!") >= 0
        True

    Test that the coding line and doctest are preserved::

        sage: Fpy = open(script_py, "r")
        sage: Fpy.readline()
        '# -*- coding: utf-8 -*-\n'
        sage: Fpy.readline()
        "'''This is a test file.\n"
        sage: Fpy.readline()
        "And I am its doctest'''\n"

    Now for a file which should fail tests::

        sage: s = s.replace('4', '5') # (2+2 != 5)
        sage: F = open(script, 'w')
        sage: _ = F.write(s)
        sage: F.close()
        sage: (out, err, ret) = test_executable([           # long time
        ....:     "sage", "-t", "--optional=sage", script])
        sage: ret                                           # long time
        1
        sage: out.find("1 item had failures:") >= 0         # long time
        True

    Test ``sage -t --debug -p 2`` on a ReST file, the ``-p 2`` should
    be ignored. In Pdb, we run the ``help`` command::

        sage: s = "::\n\n    sage: assert True is False\n    sage: 2 + 2\n    5"
        sage: script = tmp_filename(ext='.rst')
        sage: F = open(script, 'w')
        sage: _ = F.write(s)
        sage: F.close()
        sage: (out, err, ret) = test_executable([           # long time
        ....:     "sage", "-t", "--optional=sage", "--debug",
        ....:     "-p", "2", "--warn-long", "0", script], "help")
        sage: print(out)                                    # long time
        Debugging requires single-threaded operation, setting number of threads to 1.
        Running doctests with ID...
        Doctesting 1 file.
        sage -t ...
        **********************************************************************
        File "...", line 3, in ...
        Failed example:
            assert True is False
        Exception raised:
            Traceback (most recent call last):
            ...
            AssertionError
        > <doctest ...>(1)<module>()
        -> assert True is False
        (Pdb)
        Documented commands (type help <topic>):
        ========================================
        ...
        **********************************************************************
        File "...", line 4, in ...
        Failed example:
            2 + 2
        Expected:
            5
        Got:
            4
        **********************************************************************
        Previously executed commands:
            s...: assert True is False
        sage:
        <BLANKLINE>
        Returning to doctests...
        **********************************************************************
        1 item had failures:
           2 of   3 in ...
            [2 tests, 2 failures, ...]
        ...
        sage: ret                                           # long time
        1

    Now run a test for the fixdoctests script and, in particular, check that the
    issues raised in :issue:`10589` are fixed. We have to go to slightly silly
    lengths to doctest the output.::

        sage: test='r\"\"\"Add a doc-test for the fixdoctest command line option and, in particular, check that\n:issue:`10589` is fixed.\n\nEXAMPLES::\n\n    sage: 1+1              # incorrect output\n    3\n    sage: m=matrix(ZZ,3)   # output when none is expected\n    [0 0 0]\n    [0 0 0]\n    [1 0 0]\n    sage: (2/3)*m          # no output when it is expected\n    sage: mu=PartitionTuple([[4,4],[3,3,2,1],[1,1]])   # output when none is expected\n    [4, 4, 3, 3, 2, 1, 1]\n    sage: mu.pp()          # uneven indentation\n    ****\n    ****\n    sage: PartitionTuples.options(convention="French")\n    sage: mu.pp()         # fix doctest with uneven indentation\n    sage: PartitionTuples.options._reset()\n\"\"\"\n'
        sage: test_file = os.path.join(tmp_dir(), 'test_file.py')
        sage: F = open(test_file, 'w')
        sage: _ = F.write(test)
        sage: F.close()
        sage: (out, err, ret) = test_executable(["sage", "--fixdoctests", test_file])       # long time
        sage: with open(test_file, 'r') as f:                                               # long time
        ....:     fixed_test = f.read()
        sage: import difflib                                                                # long time
        sage: list(difflib.unified_diff(test.splitlines(), fixed_test.splitlines()))[2:-1]  # long time
        ['@@ -4,18 +4,23 @@\n',
         ' EXAMPLES::',
         ' ',
         '     sage: 1+1              # incorrect output',
         '-    3',
         '+    2',
         '     sage: m=matrix(ZZ,3)   # output when none is expected',
         '+    sage: (2/3)*m          # no output when it is expected',
         '     [0 0 0]',
         '     [0 0 0]',
         '-    [1 0 0]',
         '-    sage: (2/3)*m          # no output when it is expected',
         '+    [0 0 0]',
         '     sage: mu=PartitionTuple([[4,4],[3,3,2,1],[1,1]])   # output when none is expected',
         '-    [4, 4, 3, 3, 2, 1, 1]',
         '     sage: mu.pp()          # uneven indentation',
         '-    ****',
         '-    ****',
         '+       ****   ***   *',
         '+       ****   ***   *',
         '+              **',
         '+              *',
         '     sage: PartitionTuples.options(convention="French")',
         '     sage: mu.pp()         # fix doctest with uneven indentation',
         '+    *',
         '+    **',
         '+    ****   ***   *',
         '+    ****   ***   *',
         '     sage: PartitionTuples.options._reset()']

    Test external programs being called by Sage::

        sage: (out, err, ret) = test_executable(["sage", "--sh"], "echo Hello World\nexit 42\n")
        sage: out.find("Hello World\n") >= 0
        True
        sage: ret
        42

        sage: (out, err, ret) = test_executable(["sage", "--sh", "-c", "echo Hello World; exit 42"])
        sage: out.find("Hello World\n") >= 0
        True
        sage: ret
        42

        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage", "--ipython"], "\n3**33\n", pydebug_ignore_warnings=True)
        sage: out.find("5559060566555523") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: (out, err, ret) = test_executable(["sage", "--python"], "print(3^33)\n")
        sage: out
        '34\n'
        sage: err
        ''
        sage: ret
        0

        sage: (out, err, ret) = test_executable(["sage", "--python3"], "print(3^33)\n")
        sage: out
        '34\n'
        sage: err
        ''
        sage: ret
        0

        sage: (out, err, ret) = test_executable(["sage", "--cython"])
        sage: print(err)
        ...
        cython: error: cython: Need at least one source file

        sage: # needs sage.symbolic
        sage: def has_tty():
        ....:     try:
        ....:         os.open(os.ctermid(), os.O_RDONLY)
        ....:         return True
        ....:     except OSError:
        ....:         return False
        sage: (out, err, ret) = test_executable(["sage", "--ecl"], "(* 12345 54321)\n")
        sage: out.find("Embeddable Common-Lisp") >= 0
        True
        sage: out.find("670592745") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # needs sage.symbolic
        sage: (out, err, ret) = test_executable(["sage", "--lisp"], "(* 12345 54321)\n")
        sage: out.find("Embeddable Common-Lisp") >= 0
        True
        sage: out.find("670592745") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # needs sage.libs.gap
        sage: # long time
        sage: (out, err, ret) = test_executable([
        ....:     "sage", "--gap", "-q"], "Size(SymmetricGroup(5));\n")
        sage: out
        '120\n'
        sage: err.replace('gap: halving pool size.', '').strip()
        ''
        sage: ret
        0

        sage: (out, err, ret) = test_executable([                   # long time  # optional - gdb
        ....:     "sage", "--gdb"], 'quit\n')
        sage: out.find('(gdb) ') >= 0                               # long time  # optional - gdb
        True
        sage: ret                                                   # long time  # optional - gdb
        0

        sage: # needs sage.libs.eclib
        sage: (out, err, ret) = test_executable(["sage", "--mwrank", "-v0", "-q"], "0 0 0 0 1\n")
        sage: out
        'Curve [0,0,0,0,1] :\tRank = 0\n\n'
        sage: err
        ''
        sage: ret
        0

        sage: # needs sage.libs.singular
        sage: (out, err, ret) = test_executable(["sage", "--singular"], "12345*54321;\n")
        sage: out.find("A Computer Algebra System for Polynomial Computations") >= 0
        True
        sage: out.find("670592745") >= 0
        True
        sage: err
        ''
        sage: ret
        0

    Test GP using the ``-f`` option which prevents the reading of a ``.gprc``
    configuration file::

        sage: # needs sage.libs.pari
        sage: (out, err, ret) = test_executable(["sage", "--gp", "-f"], "3^33\nquit(42)\n")
        sage: out.find("PARI/GP") >= 0
        True
        sage: out.find("5559060566555523") >= 0
        True
        sage: err
        ''
        sage: ret
        42

    Some programs of which we check functionality using only ``--version``::

        sage: # needs sage.symbolic
        sage: (out, err, ret) = test_executable(["sage", "--maxima", "--version"])
        sage: out.find("Maxima ") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # optional - r
        sage: (out, err, ret) = test_executable(["sage", "--R", "--version"])
        sage: out.find("R version ") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: (out, err, ret) = test_executable(["sage", "--sqlite3", "--version"])
        sage: out.startswith("3.")
        True
        sage: err
        ''
        sage: ret
        0

    Check some things requiring an internet connection::

        sage: # optional - internet
        sage: (out, err, ret) = test_executable(["sage", "--standard"])
        sage: out.find("cython") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # optional - internet
        sage: (out, err, ret) = test_executable(["sage", "--optional"])
        sage: out.find("database_cremona_ellcurve") >= 0
        True
        sage: err
        ''
        sage: ret
        0

        sage: # optional - internet
        sage: (out, err, ret) = test_executable(["sage", "--experimental"])
        sage: out.find("valgrind") >= 0
        True
        sage: err
        ''
        sage: ret
        0

    Check an illegal command line option.  This outputs an error to stdout,
    but we allow stderr in case this changes in the future::

        sage: (out, err, ret) = test_executable(["sage", "--zzfoobar"])
        sage: (out+err).find("unknown option: --zzfoobar") >= 0
        True
        sage: ret > 0
        True

    Test ``sage --rst2ipynb file.rst`` on a ReST file::

        sage: s = "::\n\n    sage: 2^10\n    1024\n    sage: 2 + 2\n    4"
        sage: input = tmp_filename(ext='.rst')
        sage: with open(input, 'w') as F:
        ....:     _ = F.write(s)
        sage: L = ["sage", "--rst2ipynb", input]
        sage: (out, err, ret) = test_executable(L)           # optional - rst2ipynb
        sage: err                                            # optional - rst2ipynb
        ''
        sage: ret                                            # optional - rst2ipynb
        0
        sage: from json import loads                         # optional - rst2ipynb
        sage: d = loads(out)                                 # optional - rst2ipynb
        sage: sorted(d.keys())                               # optional - rst2ipynb
        ['cells', 'metadata', 'nbformat', 'nbformat_minor']
        sage: d['cells'][1]['source']                        # optional - rst2ipynb
        ['2^10']
        sage: d['cells'][2]['source']                        # optional - rst2ipynb
        ['2 + 2']

    Test ``sage --rst2ipynb file.rst file.ipynb`` on a ReST file::

        sage: s = "::\n\n    sage: 2^10\n    1024\n    sage: 2 + 2\n    4"
        sage: input = tmp_filename(ext='.rst')
        sage: output = tmp_filename(ext='.ipynb')
        sage: with open(input, 'w') as F:
        ....:     _ = F.write(s)
        sage: L = ["sage", "--rst2ipynb", input, output]
        sage: test_executable(L)                              # optional - rst2ipynb
        ('', '', 0)
        sage: import json                                     # optional - rst2ipynb
        sage: d = json.load(open(output,'r'))                 # optional - rst2ipynb
        sage: type(d)                                         # optional - rst2ipynb
        <class 'dict'>
        sage: sorted(d.keys())                                # optional - rst2ipynb
        ['cells', 'metadata', 'nbformat', 'nbformat_minor']
        sage: d['metadata']                                   # optional - rst2ipynb
        {'kernelspec': {'display_name': 'sagemath', 'name': 'sagemath'}}
        sage: d['cells'][1]['cell_type']                      # optional - rst2ipynb
        'code'

    Test ``sage --ipynb2rst file.ipynb file.rst`` on a ipynb file::

        sage: s = r'''{
        ....:  "cells": [
        ....:   {
        ....:    "cell_type": "code",
        ....:    "execution_count": 1,
        ....:    "metadata": {},
        ....:    "outputs": [
        ....:     {
        ....:      "data": {
        ....:       "text/plain": [
        ....:        "2"
        ....:       ]
        ....:      },
        ....:      "execution_count": 1,
        ....:      "metadata": {},
        ....:      "output_type": "execute_result"
        ....:     }
        ....:    ],
        ....:    "source": [
        ....:     "1+1"
        ....:    ]
        ....:   },
        ....:   {
        ....:    "cell_type": "code",
        ....:    "execution_count": null,
        ....:    "metadata": {},
        ....:    "outputs": [],
        ....:    "source": []
        ....:   }
        ....:  ],
        ....:  "metadata": {
        ....:   "kernelspec": {
        ....:    "display_name": "SageMath 8.3.beta4",
        ....:    "language": "",
        ....:    "name": "sagemath"
        ....:   },
        ....:   "language_info": {
        ....:    "codemirror_mode": {
        ....:     "name": "ipython",
        ....:     "version": 2
        ....:    },
        ....:    "file_extension": ".py",
        ....:    "mimetype": "text/x-python",
        ....:    "name": "python",
        ....:    "nbconvert_exporter": "python",
        ....:    "pygments_lexer": "ipython2",
        ....:    "version": "2.7.15"
        ....:   }
        ....:  },
        ....:  "nbformat": 4,
        ....:  "nbformat_minor": 2
        ....: }
        ....: '''
        sage: t = '.. escape-backslashes\n.. default-role:: math\n\n\n::\n\n    sage: 1+1\n    2\n\n\n\n\n'
        sage: input = tmp_filename(ext='.ipynb')
        sage: output = tmp_filename(ext='.rst')
        sage: with open(input, 'w') as F:
        ....:     _ = F.write(s)
        sage: L = ["sage", "--ipynb2rst", input, output]
        sage: _ = test_executable(L)                # long time  # optional - pandoc
        sage: print(open(output, 'r').read() == t)  # long time  # optional - pandoc  # known bug #32697
        True
    """
    pexpect_env = dict(os.environ)
    try:
        del pexpect_env["TERM"]
    except KeyError:
        pass

    __with_pydebug = hasattr(sys, 'gettotalrefcount')   # This is a Python debug build (--with-pydebug)
    if __with_pydebug and pydebug_ignore_warnings:
        pexpect_env['PYTHONWARNINGS'] = ','.join([
            'ignore::DeprecationWarning',
        ])

    kwds['encoding'] = kwds.pop('encoding', 'utf-8')

    p = Popen(args, stdin=PIPE, stdout=PIPE, stderr=PIPE, env=pexpect_env,
              **kwds)
    if input:
        p.stdin.write(input)

    p.stdin.close()
    fdout = p.stdout.fileno()
    fderr = p.stderr.fileno()
    out = []
    err = []

    while True:
        # Try reading from fdout and fderr
        rfd = []
        if fdout:
            rfd.append(fdout)
        if fderr:
            rfd.append(fderr)
        if len(rfd) == 0:
            break
        timeout = float(timeout)
        rlist = select.select(rfd, [], [], timeout)[0]

        if len(rlist) == 0:
            # Timeout!
            p.terminate()
            raise RuntimeError("timeout in test_executable()")
        if fdout in rlist:
            s = p.stdout.read(1024)
            if not s:
                fdout = None   # EOF
                p.stdout.close()
            out.append(s)
        if fderr in rlist:
            s = p.stderr.read(1024)
            if not s:
                fderr = None   # EOF
                p.stderr.close()
            err.append(s)

    # In case out or err contains a quoted string, force the use of
    # double quotes so that the output is enclosed in single
    # quotes. This avoids some doctest failures with some versions of
    # OS X and Xcode.
    out = ''.join(out)
    out = out.replace("'", '"')
    err = ''.join(err)
    err = err.replace("'", '"')

    return (out, err, p.wait())



# sage_setup: distribution = sagemath-repl
"""
This file (./numbertheory_doctest.sage) was *autogenerated* from ./numbertheory.tex,
with sagetex.sty version 2011/05/27 v2.3.1.
It contains the contents of all the sageexample environments from this file.
You should be able to doctest this file with:
sage -t ./numbertheory_doctest.sage
It is always safe to delete this file; it is not used in typesetting your
document.

Sage example in ./numbertheory.tex, line 107::

  sage: a = IntegerModRing(15)(3); b = IntegerModRing(17)(3); a, b
  (3, 3)
  sage: a == b
  False

Sage example in ./numbertheory.tex, line 135::

  sage: R = a.parent(); R
  Ring of integers modulo 15
  sage: R.characteristic()
  15

Sage example in ./numbertheory.tex, line 157::

  sage: a + a, a - 17, a * a + 1, a^3
  (6, 1, 10, 12)

Sage example in ./numbertheory.tex, line 199::

  sage: z = a.lift(); y = ZZ(a); y, type(y), y == z
  (3, <... 'sage.rings.integer.Integer'>, True)

Sage example in ./numbertheory.tex, line 228::

  sage: [Mod(x,15).additive_order() for x in range(0,15)]
  [1, 15, 15, 5, 15, 3, 5, 15, 15, 5, 3, 15, 5, 15, 15]

Sage example in ./numbertheory.tex, line 261::

  sage: [[x, Mod(x,15).multiplicative_order()]
  ....:   for x in range(1,15) if gcd(x,15) == 1]
  [[1, 1], [2, 4], [4, 2], [7, 4], [8, 4], [11, 2], [13, 4], [14, 2]]

Sage example in ./numbertheory.tex, line 276::

  sage: p = 10^20 + 39; mod(2,p).multiplicative_order()
  50000000000000000019
  sage: mod(3,p).multiplicative_order()
  100000000000000000038

Sage example in ./numbertheory.tex, line 367::

  sage: R = GF(17); [1/R(x) for x in range(1,17)]
  [1, 9, 6, 13, 7, 3, 5, 15, 2, 12, 14, 10, 4, 11, 8, 16]

Sage example in ./numbertheory.tex, line 403::

  sage: R = GF(9,name='x'); R
  Finite Field in x of size 3^2

Sage example in ./numbertheory.tex, line 409::

  sage: R.polynomial()
  x^2 + 2*x + 2

Sage example in ./numbertheory.tex, line 423::

  sage: Set([r for r in R])
  {0, 1, 2, x, x + 1, x + 2, 2*x, 2*x + 1, 2*x + 2}

Sage example in ./numbertheory.tex, line 429::

  sage: Q.<x> = PolynomialRing(GF(3))
  sage: R2 = GF(9, name='x', modulus=x^2+1); R2
  Finite Field in x of size 3^2

Sage example in ./numbertheory.tex, line 463::

  sage: p = R(x+1); R2(p)
  Traceback (most recent call last):
    ...
  TypeError: unable to coerce from a finite field other than the prime subfield

Sage example in ./numbertheory.tex, line 548::

  sage: rational_reconstruction(411,1000)
  -13/17
  sage: rational_reconstruction(409,1000)
  Traceback (most recent call last):
    ...
  ArithmeticError: rational reconstruction of 409 (mod 1000) does not exist

Sage example in ./numbertheory.tex, line 571::

  sage: def harmonic(n):
  ....:    return add([1/x for x in range(1,n+1)])

Sage example in ./numbertheory.tex, line 593::

  sage: def harmonic_mod(n, m):
  ....:     return add([1/x % m for x in range(1,n+1)])
  sage: def harmonic2(n):
  ....:     q = lcm(range(1,n+1))
  ....:     pmax = RR(q*(log(n)+1))
  ....:     m = ZZ(2*pmax^2)
  ....:     m = integer_ceil(m / q) * q + 1
  ....:     a = harmonic_mod(n,m)
  ....:     return rational_reconstruction(a,m)

Sage example in ./numbertheory.tex, line 707::

  sage: a = 2; b = 3; m = 5; n = 7; lambda0 = (b-a)/m % n; a + lambda0 * m
  17
  sage: crt(2,3,5,7)
  17

Sage example in ./numbertheory.tex, line 726::

  sage: def harmonic3(n):
  ....:     q = lcm(range(1,n+1))
  ....:     pmax = RR(q*(log(n)+1))
  ....:     B = ZZ(2*pmax^2)
  ....:     a = 0; m = 1; p = 2^63
  ....:     while m < B:
  ....:        p = next_prime(p)
  ....:        b = harmonic_mod(n,p)
  ....:        a = crt(a,b,m,p)
  ....:        m = m*p
  ....:     return rational_reconstruction(a,m)
  sage: harmonic(100) == harmonic3(100)
  True

Sage example in ./numbertheory.tex, line 755::

  sage: crt(15,1,30,4)
  45
  sage: crt(15,2,30,4)
  Traceback (most recent call last):
    ...
  ValueError: no solution to crt problem since gcd(30,4) does not divide 15-2

Sage example in ./numbertheory.tex, line 1008::

  sage: [560 % (x-1) for x in [3,11,17]]
  [0, 0, 0]

Sage example in ./numbertheory.tex, line 1226::

  sage: p = 10^10+19; a = mod(17,p); a.log(2)
  6954104378
  sage: mod(2,p)^6954104378
  17
"""



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.symbolic
"""
Benchmarks

COMMENTS:

Taken as a whole these benchmarks suggest that by far the fastest math
software is MAGMA, Mathematica, and Sage (with appropriate tuning and
choices -- it can also be very slow !).  Maxima is very slow, at least
in the form that comes with Sage (perhaps this is because of using
clisp, at least partly).  GP is slow at some thing and very fast at
others.

TESTS::

    sage: import sage.tests.benchmark
"""
from cysignals.alarm import alarm, cancel_alarm, AlarmInterrupt
from sage.combinat.combinat import fibonacci
from sage.functions.other import factorial
from sage.interfaces.gp import gp
from sage.interfaces.magma import magma, Magma
from sage.interfaces.maple import maple
from sage.interfaces.mathematica import mathematica
from sage.interfaces.maxima import maxima
from sage.interfaces.singular import singular
from sage.libs.pari import pari
from sage.matrix.matrix_space import MatrixSpace
from sage.misc.functional import log
from sage.misc.lazy_import import lazy_import
from sage.misc.timing import cputime, walltime
from sage.modular.modsym.modsym import ModularSymbols
from sage.rings.complex_mpfr import ComplexField
from sage.rings.finite_rings.finite_field_constructor import GF
from sage.rings.finite_rings.integer_mod_ring import Integers
from sage.rings.rational_field import QQ
from sage.rings.integer import Integer
from sage.rings.integer_ring import ZZ
from sage.rings.polynomial.polynomial_ring_constructor import PolynomialRing
from sage.schemes.elliptic_curves.constructor import EllipticCurve

lazy_import('sage.interfaces.macaulay2', 'macaulay2')
lazy_import('sage.libs.gap.libgap', 'libgap')


def avg(X):
    """
    Return the average of the list X.

    EXAMPLES::

        sage: from sage.tests.benchmark import avg
        sage: avg([1,2,3])
        2.0
    """
    s = sum(X, 0)
    return s / float(len(X))


STD_SYSTEMS = ['sage', 'maxima', 'libgap', 'gp', 'pari', 'python']
OPT_SYSTEMS = ['magma', 'macaulay2', 'maple', 'mathematica']


class Benchmark:
    """
    A class for running specific benchmarks against different systems.

    In order to implement an extension of this class, one must write
    functions named after the different systems one wants to test. These
    functions must perform the same task for each function. Calling
    the run command with a list of systems will then show the timings.

    EXAMPLES::

        sage: from sage.tests.benchmark import Benchmark
        sage: B = Benchmark()
        sage: def python():
        ....:     t = cputime()
        ....:     2+2
        ....:     return cputime(t)
        sage: B.python = python
        sage: B.run(systems=['python'])
        sage.tests.benchmark.Benchmark instance
          System      min         avg         max         trials          cpu or wall
        * python ...
    """
    def run(self, systems=None, timeout=60, trials=1, sort=False, optional=False):
        """
        Run the benchmarking functions for the current benchmark on the systems
        given.

        INPUT:

        - ``systems`` -- (optional) list of strings of which systems to run tests on;
          if ``None``, runs the standard systems
        - ``timeout`` -- integer (default: 60); how long (in seconds)
          to run each test for
        - ``trials`` -- integer (default: 1); number of trials
        - ``sort`` -- boolean (default: ``False``); whether to sort system names
        - ``optional`` -- boolean (default: ``False``); if systems is ``None``,
          whether to test optional systems

        EXAMPLES::

            sage: from sage.tests.benchmark import PolyFactor
            sage: PolyFactor(10, QQ).run()
            Factor a product of 2 polynomials of degree 10 over Rational Field.
              System      min         avg         max         trials          cpu or wall
            * sage        ...
            * gp          ...
        """
        if sort:
            systems.sort()
        print('\n\n\n' + str(self))
        print('  %-12s%-12s%-12s%-12s%-12s%15s' % ('System', 'min',
                                                   'avg', 'max',
                                                   'trials', 'cpu or wall'))
        if systems is None:
            systems = STD_SYSTEMS
            if optional:
                systems += OPT_SYSTEMS
        for S in systems:
            try:
                X = []
                wall = False
                for i in range(trials):
                    alarm(timeout)
                    t = getattr(self, S)()
                    cancel_alarm()
                    if isinstance(t, tuple):
                        wall = True
                        t = t[1]
                    X.append(t)
                mn = min(X)
                mx = max(X)
                av = avg(X)
                s = '* %-12s%-12f%-12f%-12f%-12s' % (S, mn, av,
                                                     mx, trials)
                if wall:
                    s += '%15fw' % t
                else:
                    s += '%15fc' % t
                print(s)
            except AlarmInterrupt:
                print('%-12sinterrupted (timeout: %s seconds wall time)' %
                      (S, timeout))
            except AttributeError:
                pass
            except Exception as msg:
                print(msg)

    bench = run

    def __repr__(self):
        """
        Print representation of self, simply coming from self.repr_str.

        EXAMPLES::

            sage: from sage.tests.benchmark import Benchmark
            sage: B = Benchmark()
            sage: B.repr_str = 'spam'
            sage: B
            spam
        """
        try:
            return self.repr_str
        except AttributeError:
            return 'sage.tests.benchmark.Benchmark instance'


class Divpoly(Benchmark):
    def __init__(self, n):
        """
        Class for benchmarking computation of the division polynomial
        of the following elliptic curve.

        EXAMPLES::

            sage: E = EllipticCurve([1,2,3,4,5])
            sage: E.division_polynomial(3)
            3*x^4 + 9*x^3 + 33*x^2 + 87*x + 35
            sage: E.division_polynomial(5)
            5*x^12 + 45*x^11 + 422*x^10 + ... - 426371

            sage: from sage.tests.benchmark import Divpoly
            sage: B = Divpoly(99)
            sage: B
            99-Division polynomial
        """
        self.__n = n
        self.repr_str = "%s-Division polynomial" % self.__n

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import Divpoly
            sage: B = Divpoly(3)
            sage: isinstance(B.sage(), float)
            True
        """
        n = self.__n
        t = cputime()
        E = EllipticCurve([1, 2, 3, 4, 5])
        E.division_polynomial(n)
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import Divpoly
            sage: B = Divpoly(3)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        n = self.__n
        t = magma.cputime()
        magma('DivisionPolynomial(EllipticCurve([1,2,3,4,5]), %s)' % n)
        return magma.cputime(t)


class PolySquare(Benchmark):
    def __init__(self, n, R):
        self.__n = n
        self.__R = R
        self.repr_str = 'Square a polynomial of degree %s over %s' % (self.__n, self.__R)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import PolySquare
            sage: B = PolySquare(3, QQ)
            sage: isinstance(B.sage(), float)
            True
        """
        R = self.__R
        n = self.__n
        f = R['x'](range(1, n + 1))
        t = cputime()
        f**2
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import PolySquare
            sage: B = PolySquare(3, QQ)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        R = magma(self.__R)
        f = magma('PolynomialRing(%s)![1..%s]' % (R.name(), self.__n))
        t = magma.cputime()
        f * f
        return magma.cputime(t)

    def maple(self):
        """
        Time the computation in Maple.

        EXAMPLES::

            sage: from sage.tests.benchmark import PolySquare
            sage: B = PolySquare(3, QQ)
            sage: isinstance(B.maple()[1], float) # optional - maple
            True
        """
        R = self.__R
        if not (R == ZZ or R == QQ):
            raise NotImplementedError
        n = self.__n
        f = maple(str(R['x'](range(1, n + 1))))
        t = walltime()
        f * f
        return False, walltime(t)


class MPolynomialPower(Benchmark):
    def __init__(self, nvars=2, exp=10, base=QQ, allow_singular=True):
        self.nvars = nvars
        self.exp = exp
        self.base = base
        self.allow_singular = allow_singular
        s = 'Compute (x_0 + ... + x_%s)^%s over %s' % (
            self.nvars - 1, self.exp, self.base)
        if self.allow_singular:
            s += ' (use singular for Sage mult.)'
        self.repr_str = s

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialPower
            sage: B = MPolynomialPower()
            sage: isinstance(B.sage()[1], float)
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        z = sum(R.gens())
        if self.allow_singular:
            z = singular(sum(R.gens()))
            t = walltime()
            z**self.exp
            return False, walltime(t)
        t = cputime()
        z**self.exp
        return cputime(t)

    def macaulay2(self):
        """
        Time the computation in Macaulay2.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialPower
            sage: B = MPolynomialPower()
            sage: isinstance(B.macaulay2()[1], float) # optional - macaulay2
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        z = macaulay2(sum(R.gens()))
        t = walltime()
        z**self.exp
        return False, walltime(t)

    def maxima(self):
        """
        Time the computation in Maxima.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialPower
            sage: B = MPolynomialPower()
            sage: isinstance(B.maxima()[1], float)
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        z = maxima(str(sum(R.gens())))
        w = walltime()
        (z**self.exp).expand()
        return False, walltime(w)

    def maple(self):
        """
        Time the computation in Maple.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialPower
            sage: B = MPolynomialPower()
            sage: isinstance(B.maple()[1], float)  # optional - maple
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        z = maple(str(sum(R.gens())))
        w = walltime()
        (z**self.exp).expand()
        return False, walltime(w)

    def mathematica(self):
        """
        Time the computation in Mathematica.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialPower
            sage: B = MPolynomialPower()
            sage: isinstance(B.mathematica()[1], float) # optional - mathematica
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        z = mathematica(str(sum(R.gens())))
        w = walltime()
        (z**self.exp).Expand()
        return False, walltime(w)

# this doesn't really expand out -- pari has no function to do so,
# as far as I know.
#     def gp(self):
#         R = PolynomialRing(self.base, self.nvars)
#         z = gp(str(sum(R.gens())))
#         gp.eval('gettime')
#         z**self.exp
#         return float(gp.eval('gettime/1000.0'))

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialPower
            sage: B = MPolynomialPower()
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        R = magma.PolynomialRing(self.base, self.nvars)
        z = R.gen(1)
        for i in range(2, self.nvars + 1):
            z += R.gen(i)
        t = magma.cputime()
        z**magma(self.exp)
        return magma.cputime(t)


class MPolynomialMult(Benchmark):
    def __init__(self, nvars=2, base=QQ, allow_singular=True):
        if nvars % 2:
            nvars += 1
        self.nvars = nvars
        self.base = base
        self.allow_singular = allow_singular
        s = 'Compute (x_0 + ... + x_%s) * (x_%s + ... + x_%s) over %s' % (
            self.nvars // 2 - 1, self.nvars // 2, self.nvars, self.base)
        if self.allow_singular:
            s += ' (use singular for Sage mult.)'
        self.repr_str = s

    def maxima(self):
        """
        Time the computation in Maxima.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult
            sage: B = MPolynomialMult()
            sage: isinstance(B.maxima()[1], float)
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = maxima(str(sum(R.gens()[:k])))
        z1 = maxima(str(sum(R.gens()[k:])))
        w = walltime()
        (z0 * z1).expand()
        return False, walltime(w)

    def maple(self):
        """
        Time the computation in Maple.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult
            sage: B = MPolynomialMult()
            sage: isinstance(B.maple()[1], float)  # optional - maple
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = maple(str(sum(R.gens()[:k])))
        z1 = maple(str(sum(R.gens()[k:])))
        w = walltime()
        (z0 * z1).expand()
        return False, walltime(w)

    def mathematica(self):
        """
        Time the computation in Mathematica.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult
            sage: B = MPolynomialMult()
            sage: isinstance(B.mathematica()[1], float) # optional - mathematica
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = mathematica(str(sum(R.gens()[:k])))
        z1 = mathematica(str(sum(R.gens()[k:])))
        w = walltime()
        (z0 * z1).Expand()
        return False, walltime(w)

#     def gp(self):
#         R = PolynomialRing(self.base, self.nvars)
#         k = self.nvars // 2
#         z0 = gp(str(sum(R.gens()[:k])))
#         z1 = gp(str(sum(R.gens()[k:])))
#         gp.eval('gettime')
#         z0*z1
#         return float(gp.eval('gettime/1000.0'))

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult
            sage: B = MPolynomialMult()
            sage: isinstance(B.sage()[1], float)
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = sum(R.gens()[:k])
        z1 = sum(R.gens()[k:])
        if self.allow_singular:
            z0 = singular(z0)
            z1 = singular(z1)
            t = walltime()
            z0 * z1
            return False, walltime(t)
        else:
            t = cputime()
            z0 * z1
            return cputime(t)

    def macaulay2(self):
        """
        Time the computation in Macaulay2.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult
            sage: B = MPolynomialMult()
            sage: isinstance(B.macaulay2()[1], float) # optional - macaulay2
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = macaulay2(sum(R.gens()[:k]))
        z1 = macaulay2(sum(R.gens()[k:]))
        t = walltime()
        z0 * z1
        return False, walltime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult
            sage: B = MPolynomialMult()
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        R = magma.PolynomialRing(self.base, self.nvars)
        z0 = R.gen(1)
        k = self.nvars // 2
        for i in range(2, k + 1):
            z0 += R.gen(i)
        z1 = R.gen(k + 1)
        for i in range(k + 1, self.nvars + 1):
            z1 += R.gen(i)
        t = magma.cputime()
        z0 * z1
        return magma.cputime(t)


class MPolynomialMult2(Benchmark):
    def __init__(self, nvars=2, base=QQ, allow_singular=True):
        if nvars % 2:
            nvars += 1
        self.nvars = nvars
        self.base = base
        self.allow_singular = allow_singular
        s = 'Compute (x_1 + 2*x_2 + 3*x_3 + ... + %s*x_%s) * (%s * x_%s + ... + %s*x_%s) over %s' % (
            self.nvars // 2, self.nvars // 2, self.nvars // 2 + 1, self.nvars // 2 + 1,
            self.nvars + 1, self.nvars + 1, self.base)
        if self.allow_singular:
            s += ' (use singular for Sage mult.)'
        self.repr_str = s

#     def gp(self):
#         R = PolynomialRing(self.base, self.nvars)
#         k = self.nvars // 2
#         z0 = R(0)
#         z1 = R(0)
#         for i in range(k):
#             z0 += (i+1)*R.gen(i)
#         for i in range(k,self.nvars):
#             z1 += (i+1)*R.gen(i)
#         z0 = gp(str(z0))
#         z1 = gp(str(z1))
#         gp.eval('gettime')
#         z0*z1
#         return float(gp.eval('gettime/1000.0'))

    def maxima(self):
        """
        Time the computation in Maxima.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult2
            sage: B = MPolynomialMult2()
            sage: isinstance(B.maxima()[1], float)
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = R(0)
        z1 = R(0)
        for i in range(k):
            z0 += (i + 1) * R.gen(i)
        for i in range(k, self.nvars):
            z1 += (i + 1) * R.gen(i)
        z0 = maxima(str(z0))
        z1 = maxima(str(z1))
        w = walltime()
        (z0 * z1).expand()
        return False, walltime(w)

    def macaulay2(self):
        """
        Time the computation in Macaulay2.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult2
            sage: B = MPolynomialMult2()
            sage: isinstance(B.macaulay2()[1], float) # optional - macaulay2
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = R(0)
        z1 = R(0)
        for i in range(k):
            z0 += (i + 1) * R.gen(i)
        for i in range(k, self.nvars):
            z1 += (i + 1) * R.gen(i)
        z0 = macaulay2(z0)
        z1 = macaulay2(z1)
        t = walltime()
        z0 * z1
        return False, walltime(t)

    def maple(self):
        """
        Time the computation in Maple.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult2
            sage: B = MPolynomialMult2()
            sage: isinstance(B.maple()[1], float) # optional - maple
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = R(0)
        z1 = R(0)
        for i in range(k):
            z0 += (i + 1) * R.gen(i)
        for i in range(k, self.nvars):
            z1 += (i + 1) * R.gen(i)
        z0 = maple(str(z0))
        z1 = maple(str(z1))
        w = walltime()
        (z0 * z1).expand()
        return False, walltime(w)

    def mathematica(self):
        """
        Time the computation in Mathematica.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult2
            sage: B = MPolynomialMult2()
            sage: isinstance(B.mathematica()[1], float) # optional - mathematica
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = R(0)
        z1 = R(0)
        for i in range(k):
            z0 += (i + 1) * R.gen(i)
        for i in range(k, self.nvars):
            z1 += (i + 1) * R.gen(i)
        z0 = mathematica(str(z0))
        z1 = mathematica(str(z1))
        w = walltime()
        (z0 * z1).Expand()
        return False, walltime(w)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult2
            sage: B = MPolynomialMult2()
            sage: isinstance(B.sage()[1], float)
            True
        """
        R = PolynomialRing(self.base, self.nvars, 'x')
        k = self.nvars // 2
        z0 = R(0)
        z1 = R(0)
        for i in range(k):
            z0 += (i + 1) * R.gen(i)
        for i in range(k, self.nvars):
            z1 += (i + 1) * R.gen(i)
        if self.allow_singular:
            z0 = singular(z0)
            z1 = singular(z1)
            t = walltime()
            z0 * z1
            return False, walltime(t)
        else:
            t = cputime()
            z0 * z1
            return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import MPolynomialMult2
            sage: B = MPolynomialMult2()
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        R = magma.PolynomialRing(self.base, self.nvars)
        z0 = R.gen(1)
        k = self.nvars // 2
        for i in range(2, k + 1):
            z0 += magma(i) * R.gen(i)
        z1 = R.gen(k + 1)
        for i in range(k + 1, self.nvars + 1):
            z1 += magma(i) * R.gen(i)
        t = magma.cputime()
        z0 * z1
        return magma.cputime(t)


class CharPolyTp(Benchmark):
    def __init__(self, N=37, k=2, p=2, sign=1):
        self.N = N
        self.k = k
        self.p = p
        self.sign = sign
        self.repr_str = "Compute the charpoly (given the matrix) of T_%s on S_%s(Gamma_0(%s)) with sign %s." % (self.p, self.k, self.N, self.sign)

    def matrix(self):
        try:
            return self._matrix
        except AttributeError:
            self._matrix = ModularSymbols(group=self.N, weight=self.k, sign=self.sign).T(self.p).matrix()
        return self._matrix

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import CharPolyTp
            sage: B = CharPolyTp()
            sage: isinstance(B.sage(), float)
            True
        """
        m = self.matrix()
        t = cputime()
        m.charpoly('x')
        return cputime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import CharPolyTp
            sage: B = CharPolyTp()
            sage: isinstance(B.gp(), float)
            True
        """
        m = gp(self.matrix())
        gp.eval('gettime')
        m.charpoly('x')
        return float(gp.eval('gettime/1000.0'))

    def pari(self):
        """
        Time the computation in Pari.

        EXAMPLES::

            sage: from sage.tests.benchmark import CharPolyTp
            sage: B = CharPolyTp()
            sage: isinstance(B.pari(), float)
            True
        """
        m = pari(self.matrix())
        t = cputime()
        m.charpoly('x')
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import CharPolyTp
            sage: B = CharPolyTp()
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        m = magma(self.matrix())
        t = magma.cputime()
        m.CharacteristicPolynomial()
        return magma.cputime(t)


class PolyFactor(Benchmark):
    def __init__(self, n, R):
        self.__n = n
        self.__R = R
        self.repr_str = "Factor a product of 2 polynomials of degree %s over %s." % (self.__n, self.__R)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import PolyFactor
            sage: B = PolyFactor(3, QQ)
            sage: isinstance(B.sage(), float)
            True
        """
        R = PolynomialRing(self.__R, 'x')
        f = R(range(1, self.__n + 1))
        g = R(range(self.__n + 1, 2 * (self.__n + 1)))
        h = f * g
        t = cputime()
        h.factor()
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import PolyFactor
            sage: B = PolyFactor(3, QQ)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        R = magma(self.__R)
        f = magma('PolynomialRing(%s)![1..%s]' % (R.name(), self.__n))
        g = magma('PolynomialRing(%s)![%s+1..2*(%s+1)]' % (
            R.name(), self.__n, self.__n))
        h = f * g
        t = magma.cputime()
        h.Factorization()
        return magma.cputime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import PolyFactor
            sage: B = PolyFactor(3, QQ)
            sage: isinstance(B.gp(), float)
            True
        """
        R = PolynomialRing(self.__R, 'x')
        f = R(range(1, self.__n + 1))
        g = R(range(self.__n + 1, 2 * (self.__n + 1)))
        h = f * g
        f = gp(h)
        gp.eval('gettime')
        f.factor()
        return float(gp.eval('gettime/1000.0'))


class SquareInts(Benchmark):
    def __init__(self, base=10, ndigits=10**5):
        self.__ndigits = ndigits
        self.base = base
        self.repr_str = "Square the integer %s^%s" % (self.base, self.__ndigits)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import SquareInts
            sage: B = SquareInts()
            sage: isinstance(B.sage(), float)
            True
        """
        n = Integer(self.base)**self.__ndigits
        t = cputime()
        n**2
        return cputime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import SquareInts
            sage: B = SquareInts()
            sage: isinstance(B.gp(), float)
            True
        """
        n = gp('%s^%s' % (self.base, self.__ndigits))
        gp.eval('gettime')
        n**2
        return float(gp.eval('gettime/1000.0'))

    def maxima(self):
        """
        Time the computation in Maxima.

        EXAMPLES::

            sage: from sage.tests.benchmark import SquareInts
            sage: B = SquareInts()
            sage: isinstance(B.maxima()[1], float)
            True
        """
        n = maxima('%s^%s' % (self.base, self.__ndigits))
        t = walltime()
        n**2
        return False, walltime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import SquareInts
            sage: B = SquareInts()
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        n = magma('%s^%s' % (self.base, self.__ndigits))
        t = magma.cputime()
        n**2
        return magma.cputime(t)

    def python(self):
        """
        Time the computation in Python.

        EXAMPLES::

            sage: from sage.tests.benchmark import SquareInts
            sage: B = SquareInts()
            sage: isinstance(B.python(), float)
            True
        """
        n = self.base**self.__ndigits
        t = cputime()
        n**2
        return cputime(t)

    def maple(self):
        """
        Time the computation in Maple.

        EXAMPLES::

            sage: from sage.tests.benchmark import SquareInts
            sage: B = SquareInts()
            sage: isinstance(B.maple()[1], float) # optional - maple
            True
        """
        n = maple('%s^%s' % (self.base, self.__ndigits))
        t = walltime()
        n**2
        return False, walltime(t)

    def libgap(self):
        """
        Time the computation in libGAP.

        EXAMPLES::

            sage: from sage.tests.benchmark import SquareInts
            sage: B = SquareInts()
            sage: isinstance(B.libgap()[1], float)
            True
        """
        n = libgap(self.base)**libgap(self.__ndigits)
        t = walltime()
        n**2
        return False, walltime(t)

    def mathematica(self):
        """
        Time the computation in Mathematica.

        EXAMPLES::

            sage: from sage.tests.benchmark import SquareInts
            sage: B = SquareInts()
            sage: isinstance(B.mathematica()[1], float) # optional - mathematica
            True
        """
        n = mathematica('%s^%s' % (self.base, self.__ndigits))
        t = walltime()
        n**2
        return False, walltime(t)


class MatrixSquare(Benchmark):
    def __init__(self, n, R):
        self.__n = n
        self.__R = R
        self.repr_str = 'Square a matrix of degree %s over %s' % (self.__n, self.__R)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import MatrixSquare
            sage: B = MatrixSquare(3, QQ)
            sage: isinstance(B.sage(), float)
            True
        """
        R = self.__R
        n = self.__n
        f = MatrixSpace(R, n)(list(range(n * n)))
        t = cputime()
        f**2
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import MatrixSquare
            sage: B = MatrixSquare(3, QQ)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        R = magma(self.__R)
        f = magma('MatrixAlgebra(%s, %s)![0..%s^2-1]' % (
            R.name(), self.__n, self.__n))
        t = magma.cputime()
        f * f
        return magma.cputime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import MatrixSquare
            sage: B = MatrixSquare(3, QQ)
            sage: isinstance(B.gp(), float)
            True
        """
        n = self.__n
        m = gp('matrix(%s,%s,m,n,%s*(m-1)+(n-1))' % (n, n, n))
        gp('gettime')
        m * m
        return float(gp.eval('gettime/1000.0'))

    def libgap(self):
        """
        Time the computation in libGAP.

        EXAMPLES::

            sage: from sage.tests.benchmark import MatrixSquare
            sage: B = MatrixSquare(3, QQ)
            sage: isinstance(B.libgap()[1], float)
            True
        """
        R = self.__R
        n = self.__n
        f = MatrixSpace(R, n)(list(range(n * n)))
        m = libgap(f)
        t = walltime()
        m * m
        return False, walltime(t)


class Factorial(Benchmark):
    def __init__(self, n):
        self.__n = n
        self.repr_str = "Compute the factorial of %s" % self.__n

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import Factorial
            sage: B = Factorial(10)
            sage: isinstance(B.sage(), float)
            True
        """
        t = cputime()
        factorial(self.__n)
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import Factorial
            sage: B = Factorial(10)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        t = magma.cputime()
        magma('&*[1..%s]' % self.__n)  # &* is way better than Factorial!!
        return magma.cputime(t)

    def maple(self):
        """
        Time the computation in Maple.

        EXAMPLES::

            sage: from sage.tests.benchmark import Factorial
            sage: B = Factorial(10)
            sage: isinstance(B.maple()[1], float) # optional - maple
            True
        """
        n = maple(self.__n)
        t = walltime()
        n.factorial()
        return False, walltime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import Factorial
            sage: B = Factorial(10)
            sage: isinstance(B.gp(), float)
            True
        """
        gp.eval('gettime')
        gp('%s!' % self.__n)
        return float(gp.eval('gettime/1000.0'))


class Fibonacci(Benchmark):
    def __init__(self, n):
        self.__n = n
        self.repr_str = "Compute the %s-th Fibonacci number" % self.__n

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import Fibonacci
            sage: B = Fibonacci(10)
            sage: isinstance(B.sage(), float)
            True
        """
        t = cputime()
        fibonacci(self.__n)
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import Fibonacci
            sage: B = Fibonacci(10)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        t = magma.cputime()
        magma('Fibonacci(%s)' % self.__n)
        return magma.cputime(t)

    def libgap(self):
        """
        Time the computation in libGAP.

        EXAMPLES::

            sage: from sage.tests.benchmark import Fibonacci
            sage: B = Fibonacci(10)
            sage: isinstance(B.libgap()[1], float)
            True
        """
        n = libgap(self.__n)
        t = walltime()
        n.Fibonacci()
        return False, walltime(t)

    def mathematica(self):
        """
        Time the computation in Mathematica.

        EXAMPLES::

            sage: from sage.tests.benchmark import Fibonacci
            sage: B = Fibonacci(10)
            sage: isinstance(B.mathematica()[1], float) # optional - mathematica
            True
        """
        n = mathematica(self.__n)
        t = walltime()
        n.Fibonacci()
        return False, walltime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import Fibonacci
            sage: B = Fibonacci(10)
            sage: isinstance(B.gp(), float)
            True
        """
        gp.eval('gettime')
        gp('fibonacci(%s)' % self.__n)
        return float(gp.eval('gettime/1000.0'))


class SEA(Benchmark):
    def __init__(self, p):
        self.__p = p
        self.repr_str = "Do SEA on an elliptic curve over GF(%s)" % self.__p

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import SEA
            sage: B = SEA(5)
            sage: isinstance(B.sage()[1], float)
            True
        """
        E = EllipticCurve([1, 2, 3, 4, 5])
        t = walltime()
        # Note that from pari 2.4.3, the SEA algorithm is used by the
        # pari library, but only for large primes, so for a better
        # test a prime > 2^30 should be used and not 5.  In fact
        # next_prime(2^100) works fine (<<1s).
        E.change_ring(GF(self.__p)).cardinality_pari()
        return False, walltime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import SEA
            sage: B = SEA(5)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        magma(0)
        t = magma.cputime()
        magma('#EllipticCurve([GF(%s)|1,2,3,4,5])' % (self.__p))
        return magma.cputime(t)


class MatrixKernel(Benchmark):
    def __init__(self, n, R):
        self.__n = n
        self.__R = R
        self.repr_str = 'Kernel of a matrix of degree %s over %s' % (self.__n, self.__R)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import MatrixKernel
            sage: B = MatrixKernel(3, QQ)
            sage: isinstance(B.sage(), float)
            True
        """
        R = self.__R
        n = self.__n
        f = MatrixSpace(R, n, 2 * n)(list(range(n * (2 * n))))
        t = cputime()
        f.kernel()
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import MatrixKernel
            sage: B = MatrixKernel(3, QQ)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        R = magma(self.__R)
        f = magma('RMatrixSpace(%s, %s, %s)![0..(%s*2*%s)-1]' % (
            R.name(), self.__n, 2 * self.__n, self.__n, self.__n))
        t = magma.cputime()
        f.Kernel()
        return magma.cputime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import MatrixKernel
            sage: B = MatrixKernel(3, QQ)
            sage: isinstance(B.gp(), float)
            True
        """
        n = self.__n
        m = gp('matrix(%s,%s,m,n,%s*(m-1)+(n-1))' % (n, 2 * n, n))
        gp('gettime')
        m.matker()
        return float(gp.eval('gettime/1000.0'))


class ComplexMultiply(Benchmark):
    def __init__(self, bits_prec, times):
        self.__bits_prec = bits_prec
        self.__times = times
        self.repr_str = "List of multiplies of two complex numbers with %s bits of precision %s times" % (self.__bits_prec, self.__times)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import ComplexMultiply
            sage: B = ComplexMultiply(28, 2)
            sage: isinstance(B.sage(), float)
            True
        """
        CC = ComplexField(self.__bits_prec)
        s = CC(2).sqrt() + (CC.gen() * 2).sqrt()
        t = cputime()
        [s * s for _ in range(self.__times)]
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import ComplexMultiply
            sage: B = ComplexMultiply(28, 2)
            sage: isinstance(B.magma(), float) # optional - magma
            True

        .. NOTE::

            decimal digits (despite magma docs that say bits!!)
        """
        n = int(self.__bits_prec / log(10, 2)) + 1
        CC = magma.ComplexField(n)
        s = CC(2).Sqrt() + CC.gen(1).Sqrt()
        t = magma.cputime()
        magma.eval('s := %s;' % s.name())
        magma('[s*s : i in [1..%s]]' % self.__times)
        return magma.cputime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import ComplexMultiply
            sage: B = ComplexMultiply(28, 2)
            sage: isinstance(B.gp(), float)
            True
        """
        n = int(self.__bits_prec / log(10, 2)) + 1
        gp.set_real_precision(n)
        gp.eval('s = sqrt(2) + sqrt(2*I);')
        gp.eval('gettime;')
        gp('vector(%s,i,s*s)' % self.__times)
        return float(gp.eval('gettime/1000.0'))


class ModularSymbols1(Benchmark):
    def __init__(self, N, k=2):
        self.__N = N
        self.__k = k
        self.repr_str = 'Presentation for modular symbols on Gamma_0(%s) of weight %s' % (self.__N, self.__k)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import ModularSymbols1
            sage: B = ModularSymbols1(11)
            sage: isinstance(B.sage(), float)
            True
        """
        t = cputime()
        ModularSymbols(self.__N, self.__k)
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import ModularSymbols1
            sage: B = ModularSymbols1(11)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        magma = Magma()  # new instance since otherwise modsyms are cached, and cache can't be cleared
        t = magma.cputime()
        magma('ModularSymbols(%s, %s)' % (self.__N, self.__k))
        return magma.cputime(t)


class ModularSymbolsDecomp1(Benchmark):
    def __init__(self, N, k=2, sign=1, bnd=10):
        self.N = N
        self.k = k
        self.sign = sign
        self.bnd = bnd
        self.repr_str = 'Decomposition of modular symbols on Gamma_0(%s) of weight %s and sign %s' % (self.N, self.k, self.sign)

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import ModularSymbolsDecomp1
            sage: B = ModularSymbolsDecomp1(11)
            sage: isinstance(B.sage(), float)
            True
        """
        t = cputime()
        M = ModularSymbols(self.N, self.k, sign=self.sign, use_cache=False)
        M.decomposition(self.bnd)
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import ModularSymbolsDecomp1
            sage: B = ModularSymbolsDecomp1(11)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        m = Magma()  # new instance since otherwise modsyms are cached, and cache can't be cleared
        t = m.cputime()
        m.eval('Decomposition(ModularSymbols(%s, %s, %s),%s);' % (
            self.N, self.k, self.sign, self.bnd))
        return m.cputime(t)


class EllipticCurveTraces(Benchmark):
    def __init__(self, B):
        self.B = B
        self.repr_str = "Compute all a_p for the elliptic curve [1,2,3,4,5], for p < %s" % self.B

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import EllipticCurveTraces
            sage: B = EllipticCurveTraces(11)
            sage: isinstance(B.sage(), float)
            Traceback (most recent call last):
            ...
            TypeError: ...anlist() got an unexpected keyword argument 'pari_ints'
        """
        E = EllipticCurve([1, 2, 3, 4, 5])
        t = cputime()
        E.anlist(self.B, pari_ints=True)
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import EllipticCurveTraces
            sage: B = EllipticCurveTraces(11)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        E = magma.EllipticCurve([1, 2, 3, 4, 5])
        t = magma.cputime()
        E.TracesOfFrobenius(self.B)
        return magma.cputime(t)


class EllipticCurvePointMul(Benchmark):
    def __init__(self, n):
        self.n = n
        self.repr_str = "Compute %s*(0,0) on the elliptic curve [0, 0, 1, -1, 0] over QQ" % self.n

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import EllipticCurvePointMul
            sage: B = EllipticCurvePointMul(11)
            sage: isinstance(B.sage(), float)
            True
        """
        E = EllipticCurve([0, 0, 1, -1, 0])
        P = E([0, 0])
        t = cputime()
        self.n * P
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import EllipticCurvePointMul
            sage: B = EllipticCurvePointMul(11)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        E = magma.EllipticCurve('[0, 0, 1, -1, 0]')
        P = E('[0,0]')
        t = magma.cputime()
        magma(self.n) * P
        return magma.cputime(t)

    def gp(self):
        """
        Time the computation in GP.

        EXAMPLES::

            sage: from sage.tests.benchmark import EllipticCurvePointMul
            sage: B = EllipticCurvePointMul(11)
            sage: isinstance(B.gp(), float)
            True
        """
        E = gp.ellinit('[0, 0, 1, -1, 0]')
        gp.eval('gettime')
        P = gp([0, 0])
        E.ellmul(P, self.n)
        return float(gp.eval('gettime/1000.0'))

    def pari(self):
        """
        Time the computation in Pari.

        EXAMPLES::

            sage: from sage.tests.benchmark import EllipticCurvePointMul
            sage: B = EllipticCurvePointMul(11)
            sage: isinstance(B.pari(), float)
            True
        """
        E = pari('ellinit([0, 0, 1, -1, 0])')
        pari('gettime')
        P = pari([0, 0])
        E.ellmul(P, self.n)
        return float(pari('gettime/1000.0'))


class EllipticCurveMW(Benchmark):
    def __init__(self, ainvs):
        self.ainvs = ainvs
        self.repr_str = "Compute generators for the Mordell-Weil group of the elliptic curve %s over QQ" % self.ainvs

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import EllipticCurveMW
            sage: B = EllipticCurveMW([1,2,3,4,5])
            sage: isinstance(B.sage()[1], float)
            True
        """
        E = EllipticCurve(self.ainvs)
        t = walltime()
        E.gens()
        return False, walltime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import EllipticCurveMW
            sage: B = EllipticCurveMW([1,2,3,4,5])
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        E = magma.EllipticCurve(str(self.ainvs))
        t = magma.cputime()
        E.Generators()
        return magma.cputime(t)


class FiniteExtFieldMult(Benchmark):
    def __init__(self, field, times):
        self.__times = times
        self.field = field
        self.e = field.gen()**(field.cardinality() / 3)
        self.f = field.gen()**(2 * field.cardinality() / 3)
        self.repr_str = "Multiply a^(#K/3) with a^(2*#K/3) where a == K.gen()"

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import FiniteExtFieldMult
            sage: B = FiniteExtFieldMult(GF(9, 'x'), 2)
            sage: isinstance(B.sage(), float)
            True
        """
        e = self.e
        f = self.f
        t = cputime()
        [e * f for _ in range(self.__times)]
        return cputime(t)

    def pari(self):
        """
        Time the computation in Pari.

        EXAMPLES::

            sage: from sage.tests.benchmark import FiniteExtFieldMult
            sage: B = FiniteExtFieldMult(GF(9, 'x'), 2)
            sage: isinstance(B.pari(), float)
            True
        """
        e = self.e.__pari__()
        f = self.f.__pari__()
        t = cputime()
        [e * f for _ in range(self.__times)]
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import FiniteExtFieldMult
            sage: B = FiniteExtFieldMult(GF(9, 'x'), 2)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        magma.eval('F<a> := GF(%s)' % (self.field.cardinality()))
        magma.eval('e := a^Floor(%s/3);' % (self.field.cardinality()))
        magma.eval('f := a^Floor(2*%s/3);' % (self.field.cardinality()))
        t = magma.cputime()
        magma('[e*f : i in [1..%s]]' % self.__times)
        return magma.cputime(t)


class FiniteExtFieldAdd(Benchmark):
    def __init__(self, field, times):
        self.__times = times
        self.field = field
        self.e = field.gen()**(field.cardinality() / 3)
        self.f = field.gen()**(2 * field.cardinality() / 3)
        self.repr_str = "Add a^(#K/3) to a^(2*#K/3) where a == K.gen()"

    def sage(self):
        """
        Time the computation in Sage.

        EXAMPLES::

            sage: from sage.tests.benchmark import FiniteExtFieldAdd
            sage: B = FiniteExtFieldAdd(GF(9,'x'), 2)
            sage: isinstance(B.sage(), float)
            True
        """
        e = self.e
        f = self.f
        t = cputime()
        [e + f for _ in range(self.__times)]
        return cputime(t)

    def pari(self):
        """
        Time the computation in Pari.

        EXAMPLES::

            sage: from sage.tests.benchmark import FiniteExtFieldAdd
            sage: B = FiniteExtFieldAdd(GF(9,'x'), 2)
            sage: isinstance(B.pari(), float)
            True
        """
        e = self.e.__pari__()
        f = self.f.__pari__()
        t = cputime()
        [e + f for _ in range(self.__times)]
        return cputime(t)

    def magma(self):
        """
        Time the computation in Magma.

        EXAMPLES::

            sage: from sage.tests.benchmark import FiniteExtFieldAdd
            sage: B = FiniteExtFieldAdd(GF(9,'x'), 2)
            sage: isinstance(B.magma(), float) # optional - magma
            True
        """
        magma.eval('F<a> := GF(%s)' % (self.field.cardinality()))
        magma.eval('e := a^Floor(%s/3);' % (self.field.cardinality()))
        magma.eval('f := a^Floor(2*%s/3);' % (self.field.cardinality()))
        t = magma.cputime()
        magma('[e+f : i in [1..%s]]' % self.__times)
        return magma.cputime(t)


"""
.. TODO::

   * multiply reals
   * modular degree
   * anlist
   * MW group
   * symbolic det
   * poly factor
   * multivariate poly factor
"""


def suite1():
    PolySquare(10000, QQ).run()
    PolySquare(20000, ZZ).run()
    PolySquare(50000, GF(5)).run()
    PolySquare(20000, Integers(8)).run()

    SquareInts(10, 2000000).run()

    MatrixSquare(200, QQ).run()
    MatrixSquare(50, ZZ).run()

    SquareInts(10, 150000).run()

    Factorial(2 * 10**6).run(systems=['sage', 'magma'])
    Fibonacci(10**6).run()
    Fibonacci(2 * 10**7).run(systems=["sage", "magma", "mathematica"])

    MatrixKernel(150, QQ).run()

    ComplexMultiply(100000, 1000)
    ComplexMultiply(100, 100000)
    ComplexMultiply(53, 100000)

    PolyFactor(300, ZZ)
    PolyFactor(300, GF(19))
    PolyFactor(700, GF(19))

    PolyFactor(500, GF((9, 2), 'a'))
    PolyFactor(100, GF((10007, 3), 'a'))

    CharPolyTp(54, 4).run()
    CharPolyTp(389, 2).run()
    CharPolyTp(389, 2, sign=0, p=3).run()
    CharPolyTp(1000, 2, sign=1, p=2).run(systems=['sage', 'magma'])
    CharPolyTp(1, 100, sign=1, p=5).run(systems=['sage', 'magma'])   # Sage's multimodular really sucks here! (GP is way better, even)
    CharPolyTp(512, sign=1, p=3).run(systems=['sage', 'magma', 'gp'])
    CharPolyTp(512, sign=0, p=3).run(systems=['sage', 'magma', 'gp'])
    CharPolyTp(1024, sign=1, p=3).run(systems=['sage', 'magma', 'gp'])
    CharPolyTp(2006, sign=1, p=2).run(systems=['sage', 'magma', 'gp'])
    CharPolyTp(2006, sign=1, p=2).run(systems=['sage', 'magma'])    # gp takes > 1 minute.


def mpoly():
    # This includes a maxima benchmark.  Note that
    # maxima is *shockingly* slow in comparison to Singular or MAGMA.
    # It is so slow as to be useless, basically, i.e., factor
    # of 5000 slower than Singular on this example!
    MPolynomialPower(nvars=6, exp=10).run()

    main = ['sage', 'magma']  # just the main competitors
    MPolynomialPower(nvars=2, exp=200, allow_singular=False).run(main)
    MPolynomialPower(nvars=5, exp=10, allow_singular=False).run(main)
    MPolynomialPower(nvars=5, exp=30, allow_singular=True).run(main)
    MPolynomialPower(nvars=2, exp=1000, allow_singular=True).run(main)
    MPolynomialPower(nvars=10, exp=10, allow_singular=True).run(main)
    MPolynomialPower(nvars=4, exp=350, base=GF(7), allow_singular=True).run(main)
    MPolynomialMult(200, allow_singular=False).run(main)
    MPolynomialMult(400, allow_singular=True).run(main)
    MPolynomialMult(800, allow_singular=True).run(main)
    MPolynomialMult2(500, allow_singular=True).run(main)


def mpoly_all(include_maple=False):
    """
    Run benchmarks for multipoly arithmetic on all systems (except
    Maxima, since it is very very slow).  You must have mathematica,
    maple, and magma.

    .. NOTE::

       * maple is depressingly slow on these benchmarks.
       * Singular (i.e., Sage) does shockingly well.
       * mathematica is sometimes amazing.
       * macaulay2 is also quite bad (though not as bad as maple).

    EXAMPLES::

        sage: from sage.tests.benchmark import mpoly_all
        sage: mpoly_all() # not tested
        <BLANKLINE>
        ...
        ...System      min         avg         max         trials          cpu or wall
        ...
        * sage...
    """
    systems = ['sage', 'magma', 'mathematica', 'macaulay2']
    if include_maple:
        systems.append('maple')
    MPolynomialMult(200).run(systems=systems)
    MPolynomialMult(400).run(systems=systems)
    MPolynomialMult2(256).run(systems=systems)
    MPolynomialMult2(512).run(systems=systems)
    MPolynomialPower(nvars=4, exp=50).run(systems=systems)   # mathematica wins
    MPolynomialPower(nvars=10, exp=10).run(systems=systems)


def modsym_present():
    ModularSymbols1(2006, 2)
    ModularSymbols1(1, 50)
    ModularSymbols1(1, 100)
    ModularSymbols1(1, 150)
    ModularSymbols1(30, 8)
    ModularSymbols1(225, 4)
    ModularSymbols1(2, 50)
    ModularSymbols1(2, 100)


def modsym_decomp():
    ModularSymbolsDecomp1(1, 24).run()
    ModularSymbolsDecomp1(125, 2).run()
    ModularSymbolsDecomp1(389, 2).run()
    ModularSymbolsDecomp1(1, 100).run()
    ModularSymbolsDecomp1(54, 4).run()


def elliptic_curve():
    EllipticCurveTraces(100000).run()
    EllipticCurveTraces(500000).run()
    Divpoly(59).run()
    EllipticCurvePointMul(1000).run()
    EllipticCurvePointMul(2000).run()
    EllipticCurvePointMul(2500).run()      # sage is clearly using the wrong algorithm -- maybe need a balanced rep!?

    # NOTE -- Sage can also do these using Simon's program, which is
    # *way* *way* faster than MAGMA...
    EllipticCurveMW([5, 6, 7, 8, 9]).run()
    EllipticCurveMW([50, 6, 7, 8, 9]).run()
    EllipticCurveMW([1, -1, 0, -79, 289]).run(trials=1)   # rank 4
    EllipticCurveMW([0, 0, 1, -79, 342]).run(trials=1)    # rank 5  (Sage wins)



# sage_setup: distribution = sagemath-repl
"Evaluating shell scripts"

import os


class Sh:
    r"""
    Evaluates a shell script and returns the output.

    To use this from the notebook type ``sh`` at the beginning of
    the input cell.  The working directory is then the (usually
    temporary) directory where the Sage worksheet process is
    executing.
    """
    def eval(self, code, globals=None, locals=None):
        r"""
        This is difficult to test because the output goes to the
        screen rather than being captured by the doctest program, so
        the following really only tests that the command doesn't bomb,
        not that it gives the right output::

            sage: sh.eval('''echo "Hello there"\nif [ $? -eq 0 ]; then\necho "good"\nfi''') # random output
        """
        # Print out the current absolute path, which is where the code
        # will be evaluated.    Evidently, users find this comforting,
        # though I personally find it to be a bit much (William Stein).
        print(os.path.abspath('.'))
        # Evaluate the input code block.  Fortunately, os.system works
        # fine with multiline input (in contrast to subprocess.Popen).
        os.system(str(code))
        # Return '' so nothing extra (for example an unsightly None)
        # gets printed when doing %sh in the notebook.
        return ''


# Create the sh object, so that %sh mode works in the notebook.
sh = Sh()



# sage_setup: distribution = sagemath-repl
# sage.doctest: needs sage.graphs sage.modules
r"""
This file contains doctests of the article ::

    Automata and Transducers
    in the Computer Algebra System Sage

by Clemens Heuberger, Daniel Krenn, and Sara Kropf, :arxiv:`1404.7458`.

IF IT BECOMES NECESSARY TO CHANGE ANY TESTS IN THIS FILE, THERE
NEEDS TO BE A ONE-YEAR DEPRECATION PERIOD. ALSO, PLEASE INFORM:

- Clemens Heuberger <clemens.heuberger@aau.at>,
- Daniel Krenn <devel@danielkrenn.at>, AND
- Sara Kropf <sara.kropf@aau.at>

IN THIS CASE REGARDING THE CHANGES!
"""

r"""

Sage example in fsm-in-sage.tex, line 376::

    sage: NAF_Abb = Transducer([(-1, 0, 0, None), (-1, 1, 1, None), (0, 0, 0, 0),
    ....:                       (0, 1, 1, 0), (1, 0, 0, 1), (1, 2, 1, -1),
    ....:                       (2, 1, 0, 0), (2, 2, 1, 0)],
    ....:                       initial_states=[-1], final_states=[0],
    ....:                       input_alphabet=[0, 1])
    sage: NAF_Abb.state(-1).format_label=lambda: r'\mathcal{I}'
    sage: NAF_Abb.latex_options(
    ....:     coordinates = {-1: (1.5, 3),
    ....:                    0: (0, 0),
    ....:                    1: (3, 0),
    ....:                    2: (6, 0)},
    ....:     initial_where = {-1: 'above'},
    ....:     format_letter=NAF_Abb.format_letter_negative,
    ....:     loop_where = lambda x: 'below')


Sage example in fsm-in-sage.tex, line 377::

    sage: str(latex(NAF_Abb))
    '\\begin{tikzpicture}[auto, initial text=, >=latex]\n\\node[state, initial, initial where=above] (v0) at (1.500000, 3.000000) {$\\mathcal{I}$};\n\\node[state, accepting] (v1) at (0.000000, 0.000000) {$0$};\n\\node[state] (v2) at (3.000000, 0.000000) {$1$};\n\\node[state] (v3) at (6.000000, 0.000000) {$2$};\n\\path[->] (v0) edge node[rotate=63.43, anchor=south] {$0\\mid \\varepsilon$} (v1);\n\\path[->] (v0) edge node[rotate=-63.43, anchor=south] {$1\\mid \\varepsilon$} (v2);\n\\path[->] (v1) edge[loop below] node {$0\\mid 0$} ();\n\\path[->] (v1.5.00) edge node[rotate=0.00, anchor=south] {$1\\mid 0$} (v2.175.00);\n\\path[->] (v2.185.00) edge node[rotate=360.00, anchor=north] {$0\\mid 1$} (v1.355.00);\n\\path[->] (v2.5.00) edge node[rotate=0.00, anchor=south] {$1\\mid \\overline{1}$} (v3.175.00);\n\\path[->] (v3.185.00) edge node[rotate=360.00, anchor=north] {$0\\mid 0$} (v2.355.00);\n\\path[->] (v3) edge[loop below] node {$1\\mid 0$} ();\n\\end{tikzpicture}'


Sage example in fsm-in-sage.tex, line 395::

    sage: NAF1 = Transducer([('I', 0, 0, None), ('I', 1, 1, None),
    ....:                    (0, 0, 0, 0), (0, 1, 1, 0),
    ....:                    (1, 0, 0, 1), (1, 2, 1, -1),
    ....:                    (2, 1, 0, 0), (2, 2, 1, 0)],
    ....:                   initial_states=['I'], final_states=[0],
    ....:                   input_alphabet=[0, 1])


Sage example in fsm-in-sage.tex, line 422::

    sage: NAF = NAF1


Sage example in fsm-in-sage.tex, line 434::

    sage: sage.combinat.finite_state_machine.FSMOldProcessOutput = False


Sage example in fsm-in-sage.tex, line 455::

    sage: str(12.digits(base=2))
    '[0, 0, 1, 1]'


Sage example in fsm-in-sage.tex, line 460::

    sage: NAF.process(12.digits(base=2)),
    ((False, 2, [0, 0, -1]),)


Sage example in fsm-in-sage.tex, line 461::

    sage: str(NAF.process(12.digits(base=2)))
    '(False, 2, [0, 0, -1])'


Sage example in fsm-in-sage.tex, line 471::

    sage: NAF_of_12 = NAF(12.digits(base=2) + [0, 0, 0])


Sage example in fsm-in-sage.tex, line 472::

    sage: str(NAF_of_12)
    '[0, 0, -1, 0, 1, 0]'


Sage example in fsm-in-sage.tex, line 482::

    sage: NAF = NAF.with_final_word_out(0)


Sage example in fsm-in-sage.tex, line 494::

    sage: NAF_of_12 = NAF(12.digits(base=2))


Sage example in fsm-in-sage.tex, line 496::

    sage: str(NAF_of_12)
    '[0, 0, -1, 0, 1]'


Sage example in fsm-in-sage.tex, line 534::

    sage: def NAF_transition(state_from, read):
    ....:     if state_from == 'I':
    ....:         write = None
    ....:         state_to = read
    ....:         return (state_to, write)
    ....:     current = 2*read + state_from
    ....:     if current % 2 == 0:
    ....:         write = 0
    ....:     elif current % 4 == 1:
    ....:         write = 1
    ....:     else:
    ....:         write = -1
    ....:     state_to = (current - write) / 2
    ....:     return (state_to, write)


Sage example in fsm-in-sage.tex, line 544::

    sage: NAF2 = Transducer(NAF_transition,
    ....:                   initial_states=['I'],
    ....:                   final_states=[0],
    ....:                   input_alphabet=[0, 1]).with_final_word_out(0)


Sage example in fsm-in-sage.tex, line 548::

    sage: NAF == NAF2
    True


Sage example in fsm-in-sage.tex, line 549::

    sage: str(NAF==NAF2)
    'True'


Sage example in fsm-in-sage.tex, line 579::

    sage: def f(state_from, read):
    ....:     current = 3*read + state_from
    ....:     write = current % 2
    ....:     state_to = (current - write) / 2
    ....:     return (state_to, write)


Sage example in fsm-in-sage.tex, line 588::

    sage: Triple = Transducer(f, input_alphabet=[0, 1],
    ....:                     initial_states=[0],
    ....:                     final_states=[0]).with_final_word_out(0)


Sage example in fsm-in-sage.tex, line 592::

    sage: three_times_four = Triple(4.digits(base=2))


Sage example in fsm-in-sage.tex, line 593::

    sage: str(three_times_four)
    '[0, 0, 1, 1]'


Sage example in fsm-in-sage.tex, line 602::

    sage: Id = Transducer([(0, 0, 0, 0), (0, 0, 1, 1)],
    ....:                 initial_states=[0], final_states=[0],
    ....:                 input_alphabet=[0, 1])


Sage example in fsm-in-sage.tex, line 608::

    sage: prebuiltId = transducers.Identity([0, 1])


Sage example in fsm-in-sage.tex, line 620::

    sage: sage.combinat.finite_state_machine.\
    ....:     FSMOldCodeTransducerCartesianProduct = False
    sage: Combined_3n_n = Triple.cartesian_product(Id).relabeled()


Sage example in fsm-in-sage.tex, line 630::

    sage: twelve_and_four = Combined_3n_n(4.digits(base=2))


Sage example in fsm-in-sage.tex, line 631::

    sage: str(twelve_and_four)
    '[(0, 0), (0, 0), (1, 1), (1, None)]'


Sage example in fsm-in-sage.tex, line 639::

    sage: def g(read0, read1):
    ....:     return ZZ(read0) - ZZ(read1)


Sage example in fsm-in-sage.tex, line 643::

    sage: Minus = transducers.operator(g, input_alphabet=[None, -1, 0, 1])


Sage example in fsm-in-sage.tex, line 644::

    sage: latex(ZZ(None))
    0


Sage example in fsm-in-sage.tex, line 650::

    sage: prebuiltMinus = transducers.sub([-1, 0, 1])


Sage example in fsm-in-sage.tex, line 654::

    sage: latex(Combined_3n_n.state(1))
    1


Sage example in fsm-in-sage.tex, line 657::

    sage: final_word_out = Combined_3n_n.state(1).final_word_out


Sage example in fsm-in-sage.tex, line 658::

    sage: str(final_word_out)
    '[(1, None)]'


Sage example in fsm-in-sage.tex, line 663::

    sage: NAF3 = Minus(Combined_3n_n).relabeled()


Sage example in fsm-in-sage.tex, line 672::

    sage: NAF_of_12 = NAF3(12.digits(base=2))


Sage example in fsm-in-sage.tex, line 673::

    sage: str(NAF_of_12)
    '[0, 0, 0, -1, 0, 1]'


Sage example in fsm-in-sage.tex, line 736::

    sage: NAF = NAF3


Sage example in fsm-in-sage.tex, line 741::

    sage: NAF3n = NAF(Triple)


Sage example in fsm-in-sage.tex, line 749::

    sage: Combined_NAF_3n_n = NAF3n.cartesian_product(NAF).relabeled()


Sage example in fsm-in-sage.tex, line 757::

    sage: T = Minus(Combined_NAF_3n_n).relabeled()


Sage example in fsm-in-sage.tex, line 762::

    sage: str(T)
    'Transducer with 9 states'


Sage example in fsm-in-sage.tex, line 769::

    sage: expansion_of_12 = T(12.digits(base=2))


Sage example in fsm-in-sage.tex, line 772::

    sage: str(expansion_of_12)
    '[0, 0, 0, 2, 0, -1, 1]'


Sage example in fsm-in-sage.tex, line 806::

    sage: def minus(trans1, trans2):
    ....:     if trans1.word_in == trans2.word_in:
    ....:         return (trans1.word_in,
    ....:                 trans1.word_out[0] - trans2.word_out[0])
    ....:     else:
    ....:         raise LookupError


Sage example in fsm-in-sage.tex, line 815::

    sage: from itertools import zip_longest
    sage: def final_minus(state1, state2):
    ....:     return [x - y for x, y in
    ....:         zip_longest(state1.final_word_out,
    ....:                     state2.final_word_out,
    ....:                     fillvalue=0)]


Sage example in fsm-in-sage.tex, line 829::

    sage: Talternative = NAF3n.product_FiniteStateMachine(
    ....:                            NAF, minus,
    ....:                            final_function=final_minus).relabeled()


Sage example in fsm-in-sage.tex, line 845::

    sage: Talternative == T
    True


Sage example in fsm-in-sage.tex, line 846::

    sage: str(Talternative==T)
    'True'


Sage example in fsm-in-sage.tex, line 854::

    sage: for t in T.iter_states():
    ....:     other = Talternative.state(t.label())
    ....:     assert t.is_final == other.is_final
    ....:     if t.is_final:
    ....:         assert t.final_word_out == other.final_word_out


Sage example in fsm-in-sage.tex, line 872::

    sage: sage.combinat.finite_state_machine.setup_latex_preamble()


Sage example in fsm-in-sage.tex, line 888::

    sage: T.set_coordinates({
    ....:     0: (-2, 0.75),
    ....:     1: (0, -1),
    ....:     2: (-6, -1),
    ....:     3: (6, -1),
    ....:     4: (-4, 2.5),
    ....:     5: (-6, 5),
    ....:     6: (6, 5),
    ....:     7: (4, 2.5),
    ....:     8: (2, 0.75)})


Sage example in fsm-in-sage.tex, line 905::

    sage: T.latex_options(format_letter=T.format_letter_negative,
    ....:                 accepting_where={
    ....:                   0: 'right',
    ....:                   1: 'below',
    ....:                   2: 'below',
    ....:                   3: 'below',
    ....:                   4: 60,
    ....:                   5: 'above',
    ....:                   6: 'above',
    ....:                   7: 120,
    ....:                   8: 'left'},
    ....:                 accepting_show_empty=True)


Sage example in fsm-in-sage.tex, line 919::

    sage: str(latex(T))
    '\\begin{tikzpicture}[auto, initial text=, >=latex, accepting text=, accepting/.style=accepting by arrow, accepting distance=7ex]\n\\node[state, initial] (v0) at (-2.000000, 0.750000) {$0$};\n\\path[->] (v0.0.00) edge node[rotate=0.00, anchor=south] {$\\$ \\mid \\varepsilon$} ++(0.00:7ex);\n\\node[state] (v1) at (0.000000, -1.000000) {$1$};\n\\path[->] (v1.270.00) edge node[rotate=450.00, anchor=south] {$\\$ \\mid \\overline{2} 0 1$} ++(270.00:7ex);\n\\node[state] (v2) at (-6.000000, -1.000000) {$2$};\n\\path[->] (v2.270.00) edge node[rotate=450.00, anchor=south] {$\\$ \\mid 0 1$} ++(270.00:7ex);\n\\node[state] (v3) at (6.000000, -1.000000) {$3$};\n\\path[->] (v3.270.00) edge node[rotate=450.00, anchor=south] {$\\$ \\mid 0 \\overline{1} 1$} ++(270.00:7ex);\n\\node[state] (v4) at (-4.000000, 2.500000) {$4$};\n\\path[->] (v4.60.00) edge node[rotate=60.00, anchor=south] {$\\$ \\mid 1$} ++(60.00:7ex);\n\\node[state] (v5) at (-6.000000, 5.000000) {$5$};\n\\path[->] (v5.90.00) edge node[rotate=90.00, anchor=south] {$\\$ \\mid \\overline{1} 0 1$} ++(90.00:7ex);\n\\node[state] (v6) at (6.000000, 5.000000) {$6$};\n\\path[->] (v6.90.00) edge node[rotate=90.00, anchor=south] {$\\$ \\mid \\overline{1} 1$} ++(90.00:7ex);\n\\node[state] (v7) at (4.000000, 2.500000) {$7$};\n\\path[->] (v7.120.00) edge node[rotate=300.00, anchor=south] {$\\$ \\mid 1 \\overline{1} 1$} ++(120.00:7ex);\n\\node[state] (v8) at (2.000000, 0.750000) {$8$};\n\\path[->] (v8.180.00) edge node[rotate=360.00, anchor=south] {$\\$ \\mid 0 \\overline{2} 0 1$} ++(180.00:7ex);\n\\path[->] (v0) edge[loop above] node {$0\\mid 0$} ();\n\\path[->] (v0) edge node[rotate=-41.19, anchor=south] {$1\\mid 0$} (v1);\n\\path[->] (v1) edge node[rotate=360.00, anchor=south] {$0\\mid \\overline{2}$} (v2);\n\\path[->] (v1) edge node[rotate=0.00, anchor=south] {$1\\mid 2$} (v3);\n\\path[->] (v2) edge node[rotate=60.26, anchor=south] {$0\\mid 0$} (v4);\n\\path[->] (v2.95.00) edge node[rotate=90.00, anchor=south] {$1\\mid 0$} (v5.265.00);\n\\path[->] (v3.95.00) edge node[rotate=90.00, anchor=south] {$0\\mid 0$} (v6.265.00);\n\\path[->] (v3) edge node[rotate=299.74, anchor=south] {$1\\mid 0$} (v7);\n\\path[->] (v4) edge node[rotate=-41.19, anchor=south] {$0\\mid 1$} (v0);\n\\path[->] (v4) edge node[rotate=308.66, anchor=south] {$1\\mid \\overline{1}$} (v5);\n\\path[->] (v5.-85.00) edge node[rotate=90.00, anchor=north] {$0\\mid \\overline{1}$} (v2.85.00);\n\\path[->] (v5) edge node[rotate=-14.04, anchor=south] {$1\\mid 1$} (v7);\n\\path[->] (v6.-85.00) edge node[rotate=90.00, anchor=north] {$1\\mid 1$} (v3.85.00);\n\\path[->] (v6) edge node[rotate=14.04, anchor=south] {$0\\mid \\overline{1}$} (v4);\n\\path[->] (v7) edge node[rotate=51.34, anchor=south] {$0\\mid 1$} (v6);\n\\path[->] (v7) edge node[rotate=41.19, anchor=south] {$1\\mid \\overline{1}$} (v8);\n\\path[->] (v8) edge node[rotate=41.19, anchor=south] {$0\\mid 0$} (v1);\n\\path[->] (v8) edge[loop above] node {$1\\mid 0$} ();\n\\end{tikzpicture}'


Sage example in fsm-in-sage.tex, line 946::

    sage: R = T.output_projection()


Sage example in fsm-in-sage.tex, line 951::

    sage: latex(len(R.states()))
    10


Sage example in fsm-in-sage.tex, line 955::

    sage: R = R.split_transitions()


Sage example in fsm-in-sage.tex, line 956::

    sage: latex(len(R.states()))
    23


Sage example in fsm-in-sage.tex, line 959::

    sage: str(R.is_deterministic())
    'False'


Sage example in fsm-in-sage.tex, line 964::

    sage: Rdet = R.determinisation()


Sage example in fsm-in-sage.tex, line 967::

    sage: latex(len(Rdet.states()))
    22


Sage example in fsm-in-sage.tex, line 974::

    sage: Rdet12 = Rdet(expansion_of_12)


Sage example in fsm-in-sage.tex, line 977::

    sage: str(Rdet12)
    'True'


Sage example in fsm-in-sage.tex, line 984::

    sage: Rdet1 = Rdet.minimization()


Sage example in fsm-in-sage.tex, line 986::

    sage: latex(len(Rdet1.states()))
    17


Sage example in fsm-in-sage.tex, line 999::

    sage: Rdet2 = R.minimization(algorithm='Brzozowski')


Sage example in fsm-in-sage.tex, line 1001::

    sage: latex(len(Rdet2.states()))
    17


Sage example in fsm-in-sage.tex, line 1034::

    sage: def weight(state_from, read):
    ....:     write = ZZ(read != 0)
    ....:     return (0, write)
    sage: Weight = Transducer(weight, input_alphabet=srange(-2, 2+1),
    ....:                     initial_states=[0], final_states=[0])


Sage example in fsm-in-sage.tex, line 1044::

    sage: prebuiltWeight = transducers.weight(srange(-2, 2+1))


Sage example in fsm-in-sage.tex, line 1050::

    sage: W = Weight(T)


Sage example in fsm-in-sage.tex, line 1051::

    sage: latex(len(W.states()))
    9


Sage example in fsm-in-sage.tex, line 1056::

    sage: W(12.digits(base=2))
    [0, 0, 0, 1, 0, 1, 1]


Sage example in fsm-in-sage.tex, line 1057::

    sage: str(W(12.digits(base=2)))
    '[0, 0, 0, 1, 0, 1, 1]'


Sage example in fsm-in-sage.tex, line 1058::

    sage: latex(add(W(12.digits(base=2))))
    3


Sage example in fsm-in-sage.tex, line 1064::

    sage: W.prepone_output()


Sage example in fsm-in-sage.tex, line 1091::

    sage: var('y')                                                                      # needs sage.symbolic
    y
    sage: def am_entry(trans):
    ....:     return y^add(trans.word_out) / 2
    sage: A = W.adjacency_matrix(entry=am_entry)                                        # needs sage.symbolic


Sage example in fsm-in-sage.tex, line 1097::

    sage: latex.matrix_column_alignment('c')


Sage example in fsm-in-sage.tex, line 1099::

    sage: latex(A)                                                                      # needs sage.symbolic
    \left(\begin{array}{ccccccccc}
    \frac{1}{2} & \frac{1}{2} \, y^{2} & 0 & 0 & 0 & 0 & 0 & 0 & 0 \\
    0 & 0 & \frac{1}{2} & \frac{1}{2} & 0 & 0 & 0 & 0 & 0 \\
    0 & 0 & 0 & 0 & \frac{1}{2} & \frac{1}{2} & 0 & 0 & 0 \\
    0 & 0 & 0 & 0 & 0 & 0 & \frac{1}{2} & \frac{1}{2} & 0 \\
    \frac{1}{2} & 0 & 0 & 0 & 0 & \frac{1}{2} \, y & 0 & 0 & 0 \\
    0 & 0 & \frac{1}{2} \, y & 0 & 0 & 0 & 0 & \frac{1}{2} \, y & 0 \\
    0 & 0 & 0 & \frac{1}{2} \, y & \frac{1}{2} \, y & 0 & 0 & 0 & 0 \\
    0 & 0 & 0 & 0 & 0 & 0 & \frac{1}{2} \, y & 0 & \frac{1}{2} \\
    0 & \frac{1}{2} \, y^{2} & 0 & 0 & 0 & 0 & 0 & 0 & \frac{1}{2}
    \end{array}\right)


Sage example in fsm-in-sage.tex, line 1109::

    sage: A1mI = (A.subs(y=1) - A.parent().identity_matrix())                           # needs sage.symbolic
    sage: (pi_not_normalized,) = A1mI.left_kernel().basis()                             # needs sage.symbolic
    sage: pi = pi_not_normalized / pi_not_normalized.norm(p=1)                          # needs sage.symbolic


Sage example in fsm-in-sage.tex, line 1110::

    sage: str(pi)                                                                       # needs sage.symbolic
    '(1/9, 1/9, 1/9, 1/9, 1/9, 1/9, 1/9, 1/9, 1/9)'


Sage example in fsm-in-sage.tex, line 1117::

    sage: expected_output = derivative(A, y).subs(y=1) * vector(len(W.states())*[1])    # needs sage.symbolic


Sage example in fsm-in-sage.tex, line 1118::

    sage: latex(expected_output)                                                        # needs sage.symbolic
    \left(1,\,0,\,0,\,0,\,\frac{1}{2},\,1,\,1,\,\frac{1}{2},\,1\right)


Sage example in fsm-in-sage.tex, line 1126::

    sage: pi * expected_output                                                          # needs sage.symbolic
    5/9


Sage example in fsm-in-sage.tex, line 1127::

    sage: latex(pi * expected_output)                                                   # needs sage.symbolic
    \frac{5}{9}


Sage example in fsm-in-sage.tex, line 1129::

    sage: latex(pi * expected_output)                                                   # needs sage.symbolic
    \frac{5}{9}


Sage example in fsm-in-sage.tex, line 1145::

    sage: var('k')                                                                      # needs sage.symbolic
    k
    sage: moments = W.asymptotic_moments(k)                                             # needs sage.symbolic


Sage example in fsm-in-sage.tex, line 1155::

    sage: latex(moments['expectation'])                                                 # needs sage.symbolic
    \frac{5}{9} \, k + \mathcal{O}\left(1\right)


Sage example in fsm-in-sage.tex, line 1162::

    sage: latex(moments['variance'])                                                    # needs sage.symbolic
    \frac{44}{243} \, k + \mathcal{O}\left(1\right)


Sage example in fsm-in-sage.tex, line 1192::

    sage: expectation_binary = Id.asymptotic_moments(k)['expectation']                  # needs sage.symbolic


Sage example in fsm-in-sage.tex, line 1195::

    sage: latex(expectation_binary)                                                     # needs sage.symbolic
    \frac{1}{2} \, k + \mathcal{O}\left(1\right)


Sage example in fsm-in-sage.tex, line 1202::

    sage: expectation_NAF = Weight(NAF).asymptotic_moments(k)['expectation']            # needs sage.symbolic


Sage example in fsm-in-sage.tex, line 1205::

    sage: latex(expectation_NAF)                                                        # needs sage.symbolic
    \frac{1}{3} \, k + \mathcal{O}\left(1\right)


Sage example in fsm-in-sage.tex, line 1211::

    sage: Abs = transducers.abs([-1, 0, 1])                                             # needs sage.symbolic


Sage example in fsm-in-sage.tex, line 1216::

    sage: latex(moments['expectation'])                                                 # needs sage.symbolic
    \frac{5}{9} \, k + \mathcal{O}\left(1\right)
"""



# sage_setup: distribution = sagemath-repl
# sage.doctest: optional - p_group_cohomology
r"""
Tests for the optional ``p_group_cohomology`` package.

AUTHOR:

- Simon King

TESTS::

    sage: from pGroupCohomology import CohomologyRing

Computation of a modular cohomology ring of a prime power group in
characteristic 2, and comparison with stored result in a database::

    sage: CohomologyRing.set_workspace(tmp_dir())
    sage: H = CohomologyRing(64,14,from_scratch=True)
    sage: H.make()
    sage: CohomologyRing.set_workspace(tmp_dir())
    sage: H0 = CohomologyRing(64,14)
    sage: H.is_isomorphic(H0)
    ('1*a_2_1', '1*c_2_2', '1*c_4_4', '1*a_1_0', '1*a_1_1', '1*a_3_3')

Computation of a modular cohomology ring of a prime power group in odd
characteristic, and some algebraic constructions in the cohomology
ring::

    sage: H = CohomologyRing(27,4)
    sage: H.make()
    sage: print(H)
    <BLANKLINE>
    Cohomology ring of Extraspecial 3-group of order 27
    and exponent 9 with coefficients in GF(3)
    <BLANKLINE>
    Computation complete
    Minimal list of generators:
    [b_2_1: 2-Cocycle in H^*(M27; GF(3)),
     c_6_2: 6-Cocycle in H^*(M27; GF(3)),
     a_1_0: 1-Cocycle in H^*(M27; GF(3)),
     a_1_1: 1-Cocycle in H^*(M27; GF(3)),
     a_3_1: 3-Cocycle in H^*(M27; GF(3)),
     a_5_1: 5-Cocycle in H^*(M27; GF(3))]
    Minimal list of algebraic relations:
    [b_2_1*a_1_0,
     a_1_0*a_3_1,
     b_2_1*a_3_1,
     a_1_0*a_5_1,
     a_3_1*a_5_1]
    sage: H.5.massey_power()
    <a_3_1; 1>: 8-Cocycle in H^*(M27; GF(3))
    sage: H.5.massey_power().as_polynomial()
    '-c_6_2*a_1_0*a_1_1'
    sage: H.essential_ideal()
    a_1_0*a_1_1,
    a_1_1*a_3_1
    sage: ascii_art(H.bar_code('LowerCentralSeries')[2])        # known bug
        *
      *-*
      *-*
    *

Computation of a modular cohomology ring of a non prime power group in
characteristic 2::

    sage: H = CohomologyRing(libgap.AlternatingGroup(6),
    ....:                    GroupName='A(6)', prime=2,
    ....:                    from_scratch=True)
    sage: H.make()
    sage: print(H)
    Cohomology ring of A(6) with coefficients in GF(2)
    <BLANKLINE>
    Computation complete
    Minimal list of generators:
    [c_2_0: 2-Cocycle in H^*(A(6); GF(2)),
     b_3_0: 3-Cocycle in H^*(A(6); GF(2)),
     b_3_1: 3-Cocycle in H^*(A(6); GF(2))]
    Minimal list of algebraic relations:
    [b_3_0*b_3_1]
"""



# sage_setup: distribution = sagemath-repl
# sage.doctest: optional - sympy
r"""
TESTS:

Checking that sympy handles correctly Sage real numbers as
coefficients of polynomials (see :issue:`24380`)::

    sage: import sympy, sympy.polys
    sage: x = sympy.Symbol('x')
    sage: p = sympy.polys.Poly(x**2 - 2.0)
    sage: p
    Poly(1.0*x**2 - 2.0, x, domain='RR')
    sage: p.coeffs()
    [1.00000000000000, -2.00000000000000]
"""



# sage_setup: distribution = sagemath-repl
"""
Representations of objects
"""
# ****************************************************************************
#       Copyright (C) 2014 Volker Braun <vbraun.name@gmail.com>
#
#  Distributed under the terms of the GNU General Public License (GPL)
#  as published by the Free Software Foundation; either version 2 of
#  the License, or (at your option) any later version.
#                  https://www.gnu.org/licenses/
# ****************************************************************************

import types
from io import StringIO

from IPython.lib.pretty import (
    _safe_getattr,
    _type_pprinters,
)

from sage.repl.display.util import format_list

_baseclass_reprs = (object.__repr__,)


class ObjectReprABC:
    """
    The abstract base class of an object representer.

    .. automethod:: __call__
    """

    def __repr__(self):
        """
        Return string representation.

        OUTPUT: string

        EXAMPLES::

            sage: from sage.repl.display.fancy_repr import ObjectReprABC
            sage: ObjectReprABC()
            ObjectReprABC pretty printer
        """
        return '{0} pretty printer'.format(self.__class__.__name__)

    def __call__(self, obj, p, cycle):
        r"""
        Format object.

        INPUT:

        - ``obj`` -- anything; object to format

        - ``p`` -- PrettyPrinter instance

        - ``cycle`` -- boolean; whether there is a cycle

        OUTPUT:

        boolean; whether the representer is applicable to ``obj``. If
        ``True``, the string representation is appended to ``p``.

        EXAMPLES::

            sage: from sage.repl.display.fancy_repr import ObjectReprABC
            sage: ObjectReprABC().format_string(123)   # indirect doctest
            'Error: ObjectReprABC.__call__ is abstract'
        """
        p.text('Error: ObjectReprABC.__call__ is abstract')
        return True

    def format_string(self, obj):
        """
        For doctesting only: Directly return string.

        INPUT:

        - ``obj`` -- anything; object to format

        OUTPUT: string

        EXAMPLES::

            sage: from sage.repl.display.fancy_repr import ObjectReprABC
            sage: ObjectReprABC().format_string(123)
            'Error: ObjectReprABC.__call__ is abstract'
        """
        from sage.repl.display.pretty_print import SagePrettyPrinter
        stream = StringIO()
        p = SagePrettyPrinter(stream, 79, '\n')
        ok = self(obj, p, False)
        if ok:
            p.flush()
            return stream.getvalue()
        else:
            return '--- object not handled by representer ---'


class SomeIPythonRepr(ObjectReprABC):

    def __init__(self):
        """
        Some selected representers from IPython.

        EXAMPLES::

            sage: from sage.repl.display.fancy_repr import SomeIPythonRepr
            sage: SomeIPythonRepr()
            SomeIPythonRepr pretty printer

        .. automethod:: __call__
        """
        type_repr = _type_pprinters.copy()
        del type_repr[type]
        del type_repr[types.BuiltinFunctionType]
        del type_repr[types.FunctionType]
        del type_repr[str]
        self._type_repr = type_repr

    def __call__(self, obj, p, cycle):
        """
        Format object.

        INPUT:

        - ``obj`` -- anything; object to format

        - ``p`` -- PrettyPrinter instance

        - ``cycle`` -- boolean; whether there is a cycle

        OUTPUT:

        boolean; whether the representer is applicable to ``obj``. If
        ``True``, the string representation is appended to ``p``.

        EXAMPLES::

            sage: from sage.repl.display.fancy_repr import SomeIPythonRepr
            sage: pp = SomeIPythonRepr()
            sage: pp.format_string(set([1, 2, 3]))
            '{1, 2, 3}'
        """
        try:
            pretty_repr = self._type_repr[type(obj)]
        except KeyError:
            return False
        pretty_repr(obj, p, cycle)
        return True


class LargeMatrixHelpRepr(ObjectReprABC):
    """
    Representation including help for large Sage matrices.

    .. automethod:: __call__
    """

    def __call__(self, obj, p, cycle):
        r"""
        Format matrix.

        INPUT:

        - ``obj`` -- anything; object to format

        - ``p`` -- PrettyPrinter instance

        - ``cycle`` -- boolean; whether there is a cycle

        OUTPUT:

        boolean; whether the representer is applicable to ``obj``. If
        ``True``, the string representation is appended to ``p``.

        EXAMPLES::

            sage: # needs sage.modules
            sage: from sage.repl.display.fancy_repr import LargeMatrixHelpRepr
            sage: M = identity_matrix(40)
            sage: pp = LargeMatrixHelpRepr()
            sage: pp.format_string(M)
            "40 x 40 dense matrix over Integer Ring (use the '.str()' method...)"
            sage: pp.format_string([M, M])
            '--- object not handled by representer ---'

        Leads to::

            sage: M                                                                     # needs sage.modules
            40 x 40 dense matrix over Integer Ring (use the '.str()' method...)
            sage: [M, M]                                                                # needs sage.modules
            [40 x 40 dense matrix over Integer Ring,
             40 x 40 dense matrix over Integer Ring]
        """
        if not p.toplevel():
            # Do not print the help for matrices inside containers
            return False
        from sage.structure.element import Matrix
        if not isinstance(obj, Matrix):
            return False
        from sage.matrix.constructor import options
        if obj.nrows() <= options.max_rows() and obj.ncols() <= options.max_cols():
            return False
        p.text(
            repr(obj) + " (use the '.str()' method to see the entries)"
        )
        return True


class PlainPythonRepr(ObjectReprABC):
    """
    The ordinary Python representation.

    .. automethod:: __call__
    """

    def __call__(self, obj, p, cycle):
        r"""
        Format matrix.

        INPUT:

        - ``obj`` -- anything; object to format

        - ``p`` -- PrettyPrinter instance

        - ``cycle`` -- boolean; whether there is a cycle

        OUTPUT:

        boolean; whether the representer is applicable to ``obj``. If
        ``True``, the string representation is appended to ``p``.

        EXAMPLES::

            sage: from sage.repl.display.fancy_repr import PlainPythonRepr
            sage: pp = PlainPythonRepr()
            sage: pp.format_string(type(1))
            "<class 'sage.rings.integer.Integer'>"

        Do not swallow a trailing newline at the end of the output of
        a custom representer. Note that it is undesirable to have a
        trailing newline, and if we don't display it you can't fix
        it::

            sage: class Newline():
            ....:     def __repr__(self):
            ....:         return 'newline\n'
            sage: n = Newline()
            sage: pp.format_string(n)
            'newline\n'
            sage: pp.format_string([n, n, n])
            '[newline\n, newline\n, newline\n]'
            sage: [n, n, n]
            [newline
             , newline
             , newline
             ]
        """
        klass = _safe_getattr(obj, '__class__', None) or type(obj)
        klass_repr = _safe_getattr(klass, '__repr__', None)
        if klass_repr in _baseclass_reprs:
            p.text(klass_repr(obj))
        else:
            # A user-provided repr. Find newlines and replace them with p.break_()
            try:
                output = repr(obj)
            except Exception:
                import sys
                import traceback
                objrepr = object.__repr__(obj).replace("object at", "at")
                exc = traceback.format_exception_only(sys.exc_info()[0], sys.exc_info()[1])
                exc = (''.join(exc)).strip()
                output = "<repr({}) failed: {}>".format(objrepr, exc)
            for idx, output_line in enumerate(output.split('\n')):
                if idx:
                    p.break_()
                p.text(output_line)
        return True


class TallListRepr(ObjectReprABC):
    """
    Special representation for lists with tall entries (e.g. matrices).

    .. automethod:: __call__
    """

    def __call__(self, obj, p, cycle):
        r"""
        Format list/tuple.

        INPUT:

        - ``obj`` -- anything; object to format

        - ``p`` -- PrettyPrinter instance

        - ``cycle`` -- boolean; whether there is a cycle

        OUTPUT:

        boolean; whether the representer is applicable to ``obj``. If
        ``True``, the string representation is appended to ``p``.

        EXAMPLES::

            sage: from sage.repl.display.fancy_repr import TallListRepr
            sage: format_list = TallListRepr().format_string
            sage: format_list([1, 2, identity_matrix(2)])                               # needs sage.modules
            '[\n      [1 0]\n1, 2, [0 1]\n]'

        Check that :issue:`18743` is fixed::

            sage: class Foo():
            ....:     def __repr__(self):
            ....:         return '''BBB    AA   RRR
            ....: B  B  A  A  R  R
            ....: BBB   AAAA  RRR
            ....: B  B  A  A  R  R
            ....: BBB   A  A  R   R'''
            ....:     def _repr_option(self, key):
            ....:         return key == 'ascii_art'
            sage: F = Foo()
            sage: [F, F]
            [
            BBB    AA   RRR    BBB    AA   RRR
            B  B  A  A  R  R   B  B  A  A  R  R
            BBB   AAAA  RRR    BBB   AAAA  RRR
            B  B  A  A  R  R   B  B  A  A  R  R
            BBB   A  A  R   R, BBB   A  A  R   R
            ]
        """
        if not (isinstance(obj, (tuple, list)) and len(obj) > 0):
            return False
        ascii_art_repr = False
        for o in obj:
            try:
                ascii_art_repr = ascii_art_repr or o._repr_option('ascii_art')
            except (AttributeError, TypeError):
                pass
            try:
                ascii_art_repr = ascii_art_repr or o.parent()._repr_option('element_ascii_art')
            except (AttributeError, TypeError):
                pass
        if not ascii_art_repr:
            return False
        output = format_list.try_format(obj)
        if output is None:
            return False
        p.text(output)
        return True



# sage_setup: distribution = sagemath-repl
##          Sage Doctest File         ##
#**************************************#
#*    Generated from PreTeXt source   *#
#*    on 2017-08-24T11:43:34-07:00    *#
#*                                    *#
#*   http://mathbook.pugetsound.edu   *#
#*                                    *#
#**************************************#
##
"""
Please contact Rob Beezer (beezer@ups.edu) with
any test failures here that need to be changed
as a result of changes accepted into Sage.  You
may edit/change this file in any sensible way, so
that development work may procede.  Your changes
may later be replaced by the authors of "Abstract
Algebra: Theory and Applications" when the text is
updated, and a replacement of this file is proposed
for review.
"""
##
## To execute doctests in these files, run
##   $ $SAGE_ROOT/sage -t <directory-of-these-files>
## or
##   $ $SAGE_ROOT/sage -t <a-single-file>
##
## Replace -t by "-tp n" for parallel testing,
##   "-tp 0" will use a sensible number of threads
##
## See: http://www.sagemath.org/doc/developer/doctesting.html
##   or run  $ $SAGE_ROOT/sage --advanced  for brief help
##
## Generated at 2017-08-24T11:43:34-07:00
## From "Abstract Algebra"
## At commit 26d3cac0b4047f4b8d6f737542be455606e2c4b4
##
## Section 1.5 Sage
##
r"""
~~~~~~~~~~~~~~~~~~~~~~ ::


~~~~~~~~~~~~~~~~~~~~~~ ::


~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: a = 10
    sage: b = 6
    sage: b = b - 10
    sage: a = a + 20
    sage: a
    30

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b = b + 50

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: b + 20
    66

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: A = matrix([[3, 1], [5,2]]); A
    [3 1]
    [5 2]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: print(A); print(); print(A.inverse())
    [3 1]
    [5 2]
    <BLANKLINE>
    [ 2 -1]
    [-5  3]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: A.   # not tested

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: A.inverse?   # not tested

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: A.inverse??   # not tested

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: B = matrix([[2, 20], [5, 50]])
    sage: B.inverse()
    Traceback (most recent call last):
    ...
    ZeroDivisionError: matrix must be nonsingular

~~~~~~~~~~~~~~~~~~~~~~ ::


~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: zoo = ['snake', 'parrot', 'elephant', 'baboon', 'beetle']
    sage: zoo
    ['snake', 'parrot', 'elephant', 'baboon', 'beetle']

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: zoo[2]
    'elephant'

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: zoo.append('ostrich'); zoo
    ['snake', 'parrot', 'elephant', 'baboon', 'beetle', 'ostrich']

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: zoo.remove('parrot')
    sage: zoo
    ['snake', 'elephant', 'baboon', 'beetle', 'ostrich']

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: mammals = zoo[1:3]
    sage: mammals
    ['elephant', 'baboon']

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: newzoo = sorted(zoo)
    sage: newzoo
    ['baboon', 'beetle', 'elephant', 'ostrich', 'snake']

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: zoo.sort()
    sage: zoo
    ['baboon', 'beetle', 'elephant', 'ostrich', 'snake']

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: plurality_zoo = [animal+'s' for animal in zoo]
    sage: plurality_zoo
    ['baboons', 'beetles', 'elephants', 'ostrichs', 'snakes']

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: dozen = srange(12); dozen
    [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: teens = srange(13, 20); teens
    [13, 14, 15, 16, 17, 18, 19]

~~~~~~~~~~~~~~~~~~~~~~ ::

    sage: decades = srange(1900, 2000, 10); decades
    [1900, 1910, 1920, 1930, 1940, 1950, 1960, 1970, 1980, 1990]
"""
